/*************************************
 * Adconverter.c
 *   
 * : 06.10.2018 11:04:47
 * :  .
 *************************************/

#include <avr/io.h>
#include <util/delay_basic.h>
#include <avr/eeprom.h>
#include "Adconverter.h"
#include "Main.h"
#include "Systemcontrol.h"
#include "Connectmode.h"

EEMEM static word Adc_AsymmOffsetvoltage=0; //    
EEMEM static word Adc_Symm1xOffsetvoltage=0; //     1x   1,1  
EEMEM static word Adc_Symm10xOffsetvoltage=0; //     10x   1,1  
EEMEM static dword Adc_ArefVoltage=ADC_AREFVOLTAGE; //    1,1  
EEMEM static dword Adc_VddrefVoltage=ADC_VDDREFVOLTAGE; //   Vdd 5  

word ADC_ReadAsymmdata(bit indxreftype_prm,byte measureinput_prm) //    
	{
	word adconvertdata_tmp=0; //  
	dword measurevoltage_tmp=0;
	switch(indxreftype_prm) //  
		{
		case FALSE: ADMUX=measureinput_prm|(ADC_BASEREFTYPE&255); //     
					break;
		case TRUE: ADMUX=measureinput_prm|(ADC_VDDREFTYPE&255); //      Vdd
		}
	_delay_loop_2(40); // 10       
	ADCSRA|=64; // 
	while((ADCSRA&16)==0); //  
	ADCSRA|=16; //  
	adconvertdata_tmp=ADCW; //  
	switch(indxreftype_prm) //  
		{
		case FALSE: measurevoltage_tmp=(dword)adconvertdata_tmp*eeprom_read_dword(&Adc_ArefVoltage)/1024; //       
					break;
		case TRUE: measurevoltage_tmp=(dword)adconvertdata_tmp*eeprom_read_dword(&Adc_VddrefVoltage)/1024; //       
		}
	return (word)measurevoltage_tmp; //    
	}

word ADC_ReadSymmdata(bit indxreftype_prm,byte measureinput_prm) //   
	{
	word adconvertdata_tmp=0; // 
	dword measurevoltage_tmp=0;  
	switch(indxreftype_prm) //  
		{
		case FALSE: ADMUX=measureinput_prm|(ADC_BASEREFTYPE&255); //      
					break; 
		case TRUE: ADMUX=measureinput_prm|(ADC_VDDREFTYPE&255); //      Vdd  
		}
	_delay_loop_2(2000); // (2000) 1       
	ADCSRA|=64; // 
	while((ADCSRA&16)==0); //  
	ADCSRA|=16; //  
	if(ADCW>511) adconvertdata_tmp=1024-ADCW; //   
	else adconvertdata_tmp=ADCW; //   
	switch(indxreftype_prm) //  
		{
		case FALSE: measurevoltage_tmp=(dword)adconvertdata_tmp*eeprom_read_dword(&Adc_ArefVoltage)/512; //       
					break;
		case TRUE: measurevoltage_tmp=(dword)adconvertdata_tmp*eeprom_read_dword(&Adc_VddrefVoltage)/512; //       
		}
	return (word)measurevoltage_tmp; //   
	}

void ADC_InitializeDevice(void) //   
	{
	word measuredata_tmp=0; // 
	ADMUX=ADC_BASEREFTYPE&255; //   1,1 
	ADCSRA=135; //   125 
	_delay_loop_2(400); //      160 
	measuredata_tmp=ADC_ReadAsymmdata(FALSE,ADCOFFSET_AGND); //    
	if((eeprom_read_word(&Adc_AsymmOffsetvoltage)==0)&&(measuredata_tmp>0)) ADC_OffsetMeasure(0); //      
	measuredata_tmp=ADC_ReadSymmdata(FALSE,ADCOFFSET1X_22INPUT); //     1   1,1  
	if((eeprom_read_word(&Adc_Symm1xOffsetvoltage)==0)&&(measuredata_tmp>0)) ADC_OffsetMeasure(1); //         1x
	measuredata_tmp=ADC_ReadSymmdata(FALSE,ADCOFFSET10X_00INPUT); //     10   1,1  
	if((eeprom_read_word(&Adc_Symm10xOffsetvoltage)==0)&&(measuredata_tmp>0)) ADC_OffsetMeasure(2); //         10x
	}

void ADC_OffsetMeasure(byte typemeasure_prm) //      
	{
	byte countcycle_tmp=0; //  
	word measuredata_tmp=0;
	while(countcycle_tmp<100) //    
		{
		switch(typemeasure_prm) //  
			{
			case 0: measuredata_tmp+=ADC_ReadAsymmdata(FALSE,ADCOFFSET_AGND); //       
					break;
			case 1: measuredata_tmp+=ADC_ReadSymmdata(FALSE,ADCOFFSET1X_22INPUT); //        1x
					break;
			case 2: measuredata_tmp+=ADC_ReadSymmdata(FALSE,ADCOFFSET10X_00INPUT); //        10x
			}
		countcycle_tmp++; //  
		SYS_TimeWaiting(100); // 100 
		}
	measuredata_tmp/=countcycle_tmp; //    
	switch(typemeasure_prm) //     
		{
		case 0: eeprom_update_word(&Adc_AsymmOffsetvoltage,measuredata_tmp); //      
				break;
		case 1: eeprom_update_word(&Adc_Symm1xOffsetvoltage,measuredata_tmp); //       1x
				break;
		case 2: eeprom_update_word(&Adc_Symm10xOffsetvoltage,measuredata_tmp); //       10x
		}
	}

word ADC_AverageAsymmdata(bit indxreftype_prm,byte numbercycle_prm,byte measureinput_prm) //     
	{
	byte countmeasure_tmp=0; // 
	dword measurevoltage_tmp=0; 
	word offsetdata_tmp=0;
	while(countmeasure_tmp<numbercycle_prm) //   
		{
		measurevoltage_tmp+=ADC_ReadAsymmdata(indxreftype_prm,measureinput_prm); //  
		offsetdata_tmp+=eeprom_read_word(&Adc_AsymmOffsetvoltage); //   
		countmeasure_tmp++; //  
		SYS_TimeWaiting(1); // 1 
		}
	if(measurevoltage_tmp>=offsetdata_tmp) measurevoltage_tmp-=offsetdata_tmp; //  
	measurevoltage_tmp/=countmeasure_tmp; //  
	return (word)measurevoltage_tmp; //  
	}

word ADC_AverageSymmdata(bit indxreftype_prm,byte numbercycle_prm,byte measureinput_prm) //     
	{
	byte countmeasure_tmp=0; // 
	dword measurevoltage_tmp=0;
	word offsetdata_tmp=0;
	while(countmeasure_tmp<numbercycle_prm) //   
		{
		measurevoltage_tmp+=ADC_ReadSymmdata(indxreftype_prm,measureinput_prm); //  
		switch(measureinput_prm) //      
			{
			case ADCSYMM1X_12INPUT:	offsetdata_tmp+=eeprom_read_word(&Adc_Symm1xOffsetvoltage); //     1
									break;
			case ADCSYMM10X_10INPUT: offsetdata_tmp+=eeprom_read_word(&Adc_Symm10xOffsetvoltage); //     10
			}
		countmeasure_tmp++; //  
		SYS_TimeWaiting(1); // 1 
		}
	if(measurevoltage_tmp>=offsetdata_tmp) measurevoltage_tmp-=offsetdata_tmp; //  
	measurevoltage_tmp/=countmeasure_tmp; //   
	return (word)measurevoltage_tmp; //   
	}

void ADC_WriteRefvoltage(bit indxreftype_prm,dword refvoltagedata_prm) //    
	{
	switch(indxreftype_prm) //  
		{
		case FALSE: eeprom_update_dword(&Adc_ArefVoltage,refvoltagedata_prm); //   
					break;
		case TRUE: eeprom_update_dword(&Adc_VddrefVoltage,refvoltagedata_prm); //    Vdd
		}
	}

dword ADC_ReadRefvoltage(bit indxreftype_prm) //     
	{
	dword refvoltagedata_tmp=0; // 
	switch(indxreftype_prm) //  
		{
		case FALSE: refvoltagedata_tmp=eeprom_read_dword(&Adc_ArefVoltage); //     
					break;
		case TRUE: refvoltagedata_tmp=eeprom_read_dword(&Adc_VddrefVoltage); //     Vdd
		}
	return refvoltagedata_tmp; //   
	}













