/**************************************************
* Cappmeasure.c
*      1 
* : 18.02.2021 18:18:08
* :  .
***************************************************/

// 
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/eeprom.h>
#include <avr/pgmspace.h>
#include <util/delay_basic.h>
#include <avr/sfr_defs.h>
#include <math.h>
#include "Main.h"
#include "Capmeasure.h"
#include "Systemcontrol.h"
#include "Symbolcharge.h"
#include "Outmessage.h"
#include "Connectmode.h"
#include "Indmeasure.h"

bit_cap indx_cap={0,0,0,0,0,0,0,0,0}; // 
var_cap alter_cap={0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0}; // 

EEMEM static dword Cap_SupportInductance=CAPSUPPORT_INDUCTANCE; //   
EEMEM static dword Cap_ExtraCapacity=CAPEXTRA_CAPACITY; //   
EEMEM static dword Cap_ProbeCapacity=CAPPROBE_CAPACITY; //     
EEMEM static dword Cap_OffsetCapacity=CAPOFFSET_CAPACITY; //    

PROGMEM const floatpoint Cap_CorrectConstants[19]={0.0855,0.1027,0.1178,0.1237,0.1992,0.333,0.4172,1.2136,2.69,2.9,5.2045,7.2446,10.29,10.69,10.85,39.7,41.0,42.0,100.0}; // 
PROGMEM const dword Cap_FrequencyRange[19]={14188,17153,21159,29732,44548,54150,61169,79597,101126,108598,118275,129469,136401,137439,139851,142275,142384,142646,143100}; //    

void CAP_ConstantaMeasure(void) //  
	{
	dword inductancedata_tmp=0; // 
	floatpoint calculationdata_tmp=0;
	inductancedata_tmp=eeprom_read_dword(&Cap_SupportInductance); //   
	alter_cap.supportcapacity=IND_ReadsupportCapacity(); //   
	calculationdata_tmp=2*M_PI*(floatpoint)inductancedata_tmp*CAPFREQU_CRYSTAL*CAPRATIO_TIMER0; //    
	alter_cap.constantadatameasure=(ldword)calculationdata_tmp; //    
	alter_cap.constantadatafrequency=CAPFREQU_CRYSTAL; //   
	alter_cap.constantadatafrequency*=CAPRATIO_TIMER0; //      
	alter_cap.constantatotalcapacity=alter_cap.supportcapacity*eeprom_read_dword(&Cap_ExtraCapacity)/(alter_cap.supportcapacity+eeprom_read_dword(&Cap_ExtraCapacity)); //   
	alter_cap.measuredatacapacity=0; //  
	alter_cap.measuredatafrequency=0;
	}

byte CAP_ReadCursorposition(bit indxplaceposition_prm) //    
	{
	byte cursorposition_tmp=0; // 
	switch(indxplaceposition_prm) // /
		{
		case FALSE: cursorposition_tmp=alter_cap.spotpositioncursor; //     
					break;
		case TRUE: cursorposition_tmp=alter_cap.linepositioncursor; //    
		}
	return cursorposition_tmp; //     
	}

void CAP_ModeMeasure(bit initializemode_prm) //   
	{
	CAP_DisplayMeasure(initializemode_prm); //   
	CAP_CycleMeasure(); //  
	}

dword CAP_DecoderLimit(ldword measuredvalue_prm,byte limitingvalue_prm) //   
	{
	ldword measuredata_tmp; // 
	measuredata_tmp=measuredvalue_prm; //  
	while(measuredata_tmp>9999) //   
		{
		if(alter_cap.countlimitmeasure<limitingvalue_prm) //    
			{
			alter_cap.countlimitmeasure++; //  
			measuredata_tmp/=10; //   10 
			}
		else break; //      
		}
	return (dword)measuredata_tmp; //  
	}

void CAP_CycleMeasure(void) //   
	{
	ldword capacitydata_tmp=0; // 
	ldword reactivedata_tmp=0;
	if((indx_cap.readydatameasure==FALSE)&&(indx_cap.startcyclemeasure==FALSE)) //   
		{
		indx_cap.startcyclemeasure=TRUE; //   
		SYS_InitializeVariable(); //   
		SYS_SwitchBlockmode(TRUE); //  
		CAP_InitializeVariable(); // 
		SYM_IndicationMeasure(TRUE); //  
		CAP_StartstopMeasure(TRUE); //  
		}
	else if((indx_cap.readydatameasure==TRUE)&&(indx_cap.startcyclemeasure==TRUE)) //   
		{
		CAP_StartstopMeasure(FALSE); //  
		indx_cap.readydatameasure=FALSE; //   
		indx_cap.startcyclemeasure=FALSE; //   
		indx_cap.displaydatameasure=TRUE; //   
		SYM_IndicationMeasure(FALSE); //  
		if(indx_cap.datatimeoverflow==FALSE) capacitydata_tmp=CAP_ProcessMeasure(); //      
		if(capacitydata_tmp>0) reactivedata_tmp=CAP_CalculationReactive(capacitydata_tmp); //       
		switch(indx_cap.capacityreactive) // /
			{
			case FALSE: alter_cap.measuredatacapacity=CAP_DecoderLimit(capacitydata_tmp,4); //   
						break;
			case TRUE:  alter_cap.measuredatareactive=CAP_DecoderLimit(reactivedata_tmp,6); //   
			}
		SYS_SwitchBlockmode(FALSE); //  
		}
	}

void CAP_DisplayMeasure(bit initializemode_prm) //   
	{
	dword extrameasuredata_tmp=0; // 
	if(indx_cap.displaydatameasure==TRUE) //      
		{
		indx_cap.displaydatameasure=FALSE; // 
		if(initializemode_prm==TRUE) indx_cap.initializecapmode=TRUE; //      
		if((indx_cap.initializecapmode==TRUE)||(alter_cap.countlimitmeasure!=alter_cap.backuplimitmeasure)) //      
			{
			CAP_InitializeLine(); //     
			}
		CAP_MeasureDataview(); //  
		switch(alter_cap.countlineposition) //  
			{
			case 0 ... 1: switch(alter_cap.countlineaddition) //   
						{
						case 0: SYM_OutdataMeasure000000((dword)alter_cap.measuredatafrequency,7,1); //  
								break;
						case 1: SYM_OutdataMeasure00_000(alter_cap.measuredataperiod,7,10,1); //  
								break;
						case 2: SYM_OutdataMeasure0_00000(alter_cap.datalossescapacity,7,9,1); //  
								break;
						case 3: SYM_OutdataMeasure00000_0(alter_cap.dataqualitycapacity,7,13,1); //  
						}
					break;
			case 2: extrameasuredata_tmp=eeprom_read_dword(&Cap_OffsetCapacity); //   
					SYM_OutdataMeasure00000_0(extrameasuredata_tmp,12,18,1); //   
					break;
			case 3: extrameasuredata_tmp=eeprom_read_dword(&Cap_SupportInductance); //  
					SYM_OutdataMeasure000_00(extrameasuredata_tmp,12,16,1); //  
					break;
			case 4: extrameasuredata_tmp=eeprom_read_dword(&Cap_ProbeCapacity); //    
					SYM_OutdataMeasure000_0(extrameasuredata_tmp,12,16,1); //    
					break;
			case 5: extrameasuredata_tmp=eeprom_read_dword(&Cap_ExtraCapacity); //   
					SYM_OutdataMeasure0000_0(extrameasuredata_tmp,12,17,1); //   
			}
		indx_cap.pinchiptypedevice=SYS_ReadtypePinchip(); //    pin/chip
		OUT_PinchipType(indx_cap.pinchiptypedevice); //   pin/chip
		if(alter_cap.countlimitmeasure>0) alter_cap.countlimitmeasure=0; //     
		}
	}

void CAP_InitializeLine(void) //      
	{
	if(indx_cap.initializecapmode==TRUE) indx_cap.initializecapmode=FALSE; //    
	if(alter_cap.countlimitmeasure!=alter_cap.backuplimitmeasure) alter_cap.backuplimitmeasure=alter_cap.countlimitmeasure; //      
	switch(indx_cap.capacityreactive) //  /
		{
		case FALSE: switch(alter_cap.backuplimitmeasure)  //    
						{
						case 0: SYM_OutcharLine(3,0,0); // : C=   ,  
								break;
						case 1: SYM_OutcharLine(3,1,0); // : C= ,    
								break;
						case 2: SYM_OutcharLine(3,2,0); // : C=  ,   
								break;
						case 3: SYM_OutcharLine(3,3,0); // : C=   ,  
								break;
						case 4: SYM_OutcharLine(3,4,0); // : C= ,    
						}
					break;
		case TRUE: switch(alter_cap.backuplimitmeasure) //     
						{
						case 0: SYM_OutcharLine(3,5,0); // : X= ,    
								break;
						case 1: SYM_OutcharLine(2,5,0); // : X=  ,   
								break;
						case 2: SYM_OutcharLine(2,6,0); // : X=   ,  
								break;
						case 3: SYM_OutcharLine(3,6,0); // : X= ,    
								break;
						case 4: SYM_OutcharLine(3,7,0); // : X=  ,   
								break;
						case 5: SYM_OutcharLine(3,8,0); // : X=   ,  
								break;
						case 6: SYM_OutcharLine(3,9,0); // : X= ,    
						}
		}
	}

void CAP_MeasureDataview(void) //    
	{
	switch(indx_cap.capacityreactive) // /
		{
		case FALSE:	switch(alter_cap.backuplimitmeasure) //   
						{
						case 0: SYM_OutdataMeasure000_0(alter_cap.measuredatacapacity,7,11,0); //   000,0
								break;
						case 1: SYM_OutdataMeasure0_000(alter_cap.measuredatacapacity,7,9,0); //   0,000
								break;
						case 2: SYM_OutdataMeasure00_00(alter_cap.measuredatacapacity,7,10,0); //   00,00
								break;
						case 3: SYM_OutdataMeasure000_0(alter_cap.measuredatacapacity,7,11,0); //   000,0
								break;
						case 4: SYM_OutdataMeasure0_000(alter_cap.measuredatacapacity,7,9,0); //   0,000
						}
					break;
		case TRUE: switch(alter_cap.backuplimitmeasure) //   
						{
						case 0: SYM_OutdataMeasure0_000(alter_cap.measuredatareactive,7,9,0); //   0,000
								break;
						case 1: SYM_OutdataMeasure00_00(alter_cap.measuredatareactive,7,10,0); //   00,00
								break;
						case 2: SYM_OutdataMeasure000_0(alter_cap.measuredatareactive,7,11,0); //   000,0
								break;
						case 3: SYM_OutdataMeasure0_000(alter_cap.measuredatareactive,7,9,0); //   0,000
								break;
						case 4: SYM_OutdataMeasure00_00(alter_cap.measuredatareactive,7,10,0); //   00,00
								break;
						case 5: SYM_OutdataMeasure000_0(alter_cap.measuredatareactive,7,11,0); //   000,0
								break;
						case 6: SYM_OutdataMeasure0_000(alter_cap.measuredatareactive,7,9,0); //   0,000
						}
		}
	}

void CAP_InitializeVariable(void) //  
	{
	if(indx_cap.readydatameasure==TRUE) indx_cap.readydatameasure=FALSE; //      
	if(indx_cap.datatimeoverflow==TRUE) indx_cap.datatimeoverflow=FALSE; //    
	if(alter_cap.countoverflowtimer>0) alter_cap.countoverflowtimer=0; //     
	if(alter_cap.countintervalmeasure>0) alter_cap.countintervalmeasure=0; //      
	}

void CAP_StartstopMeasure(bit indxstartstop_prm) //     
	{
	switch(indxstartstop_prm) // /
		{
		case FALSE: cli(); //  
					ClearBit(TIMSK0,TOIE0); //     0
					ClearBit(TIMSK1,TOIE1); //    1
					break;
		case TRUE: TCNT0=250; // 0
					TCNT1=0; // 1
					TCCR0B=6; //  0      
					SetBit(TIMSK0,TOIE0); //    0
					sei(); //  
		}
	}

void CAP_ReadMeasure(bit readydatameasure_prm,bit datatimeoverflow_prm,word countintervalmeasure_prm,dword countoverflowtimer_prm) //   
	{
	indx_cap.readydatameasure=readydatameasure_prm; //    
	indx_cap.datatimeoverflow=datatimeoverflow_prm; //  
	alter_cap.countintervalmeasure=countintervalmeasure_prm; //  
	alter_cap.countoverflowtimer=countoverflowtimer_prm; //   1
	}

ldword CAP_ProcessMeasure(void) //    
	{
	ldword offsetcapacity_tmp=0; // 
	ldword capacitydata_tmp=0;
	capacitydata_tmp=CAP_ResultsMeasure(); //   
	if(indx_cap.writeoffsetcapacity==TRUE) //     
		{
		indx_cap.writeoffsetcapacity=FALSE; // 
		if(capacitydata_tmp>=alter_cap.supportcapacity) offsetcapacity_tmp=capacitydata_tmp; //      
		else offsetcapacity_tmp=(ldword)alter_cap.supportcapacity-((ldword)alter_cap.supportcapacity-capacitydata_tmp); //      
		eeprom_update_dword(&Cap_OffsetCapacity,(dword)offsetcapacity_tmp); //  
		}
	if(indx_cap.pinchiptypedevice==TRUE) capacitydata_tmp-=eeprom_read_dword(&Cap_ProbeCapacity); //     
	if(capacitydata_tmp>(eeprom_read_dword(&Cap_OffsetCapacity)+5)) //      0,5 
		{
		if(SYS_ReadindxOnoffaudio()>0) SYS_OutsignalBeep(1); //    1 ,  
		capacitydata_tmp-=eeprom_read_dword(&Cap_OffsetCapacity); //     
		alter_cap.datalossescapacity=CAP_MeasureLosses(capacitydata_tmp); //   
		alter_cap.dataqualitycapacity=CAP_QualityCapacity(); //  
		}
	else //      
		{
		capacitydata_tmp=0; //  
		if((alter_cap.countlineposition!=1)&&(alter_cap.measuredatafrequency>0)) alter_cap.measuredatafrequency=0; //         
		if(alter_cap.datalossescapacity>0) alter_cap.datalossescapacity=0; //  
		if(alter_cap.dataqualitycapacity>0) alter_cap.dataqualitycapacity=0; //  
		}
	return capacitydata_tmp; //  
	}

ldword CAP_ResultsMeasure(void) //   
	{
	ldword capacitydata_tmp=0; // 
	ldword measuredataperiod_tmp=0;
	if(alter_cap.countoverflowtimer>0) measuredataperiod_tmp=CAPRATIO_TIMER1*alter_cap.countoverflowtimer; //  
	measuredataperiod_tmp+=(ldword)TCNT1; //   1
	alter_cap.measuredatafrequency=CAP_CalculationFrequency(measuredataperiod_tmp); //    
	alter_cap.measuredataperiod=CAP_CalculationPeriod(); //  
	alter_cap.currentdatafrequency=alter_cap.measuredatafrequency; //   
	capacitydata_tmp=CAP_CalculationCapacity(measuredataperiod_tmp); //   
	return capacitydata_tmp; //    
	}

ldword CAP_CalculationFrequency(ldword measuredataperiod_prm) //  
	{
	ldword measuredatafrequency_tmp=0; // 
	measuredatafrequency_tmp=alter_cap.constantadatafrequency; //   
	measuredatafrequency_tmp*=alter_cap.countintervalmeasure; //     
	measuredatafrequency_tmp/=measuredataperiod_prm; //  
	return measuredatafrequency_tmp; //  
	}

dword CAP_CalculationPeriod(void) //  
	{
	ldword calculationperiod_tmp=0; // 
	calculationperiod_tmp=1000000000/alter_cap.measuredatafrequency; // 
	return (dword)calculationperiod_tmp; //  
	}

ldword CAP_CalculationCapacity(ldword measuredataperiod_prm) //  
	{
	dword inductancedata_tmp=0; // 
	ldword frequencydata_tmp=0;
	ldword capacitydata_tmp=0;
	floatpoint intermediatedata_tmp=0;
	frequencydata_tmp=CAP_CalculationFrequency(measuredataperiod_prm); // 
	inductancedata_tmp=eeprom_read_dword(&Cap_SupportInductance); //   
	intermediatedata_tmp=2*M_PI*(floatpoint)frequencydata_tmp; //  
	intermediatedata_tmp*=intermediatedata_tmp; //  
	intermediatedata_tmp/=1000; //  
	intermediatedata_tmp*=(floatpoint)inductancedata_tmp; // 
	capacitydata_tmp=1000000000000000000/(ldword)intermediatedata_tmp; // 
	return capacitydata_tmp; //  
	}

dword CAP_CalculationReactive(ldword capacitydata_prm) //   
	{
	floatpoint intermediatedata_tmp=0; // 
	ldword reactivedata_tmp=0;
	intermediatedata_tmp=2*M_PI*(floatpoint)alter_cap.measuredatafrequency*(floatpoint)capacitydata_prm; //  
	reactivedata_tmp=10000000000000000/(ldword)intermediatedata_tmp; //  
	return (dword)reactivedata_tmp; //   
	}

dword CAP_MeasureLosses(ldword capacitydata_prm) //    
	{
	dword lossescapacity_tmp=0; // 
	ldword highfrequcapacity_tmp=0;
	ldword measuredataperiod_tmp=0;
	CON_SwitchDevice(TRUE,8); //  
	SYS_TimeWaiting(50); //  
	SYS_InitializeVariable(); //   
	CAP_InitializeVariable(); // 
	CAP_StartstopMeasure(TRUE); //  
	while(indx_cap.readydatameasure==FALSE); //  
	CAP_StartstopMeasure(FALSE); //  
	CON_SwitchDevice(FALSE,8); //  
	SYS_TimeWaiting(50); //  
	indx_cap.readydatameasure=FALSE; //   
	if(alter_cap.countoverflowtimer>0) measuredataperiod_tmp=CAPRATIO_TIMER1*alter_cap.countoverflowtimer; //  
	measuredataperiod_tmp+=(ldword)TCNT1; //   1
	highfrequcapacity_tmp=CAP_CalculationCapacity(measuredataperiod_tmp); //    
	lossescapacity_tmp=CAP_CalculationLosses(capacitydata_prm,highfrequcapacity_tmp); //  
	return lossescapacity_tmp; //  
	}

dword CAP_CalculationLosses(ldword capacitydata_prm,ldword highfrequcapacity_prm) //    
	{
	ldword nolossescapacity_tmp=0; // 
	floatpoint valuelossesdata_tmp=0;
	nolossescapacity_tmp=capacitydata_prm+alter_cap.constantatotalcapacity; //        
	if(indx_cap.pinchiptypedevice==TRUE) nolossescapacity_tmp+=eeprom_read_dword(&Cap_ProbeCapacity); //     
	if(nolossescapacity_tmp>highfrequcapacity_prm) //     
		{
		valuelossesdata_tmp=sqrt((floatpoint)nolossescapacity_tmp/(floatpoint)highfrequcapacity_prm-1); //  
		valuelossesdata_tmp=CAP_CorrectLosses(valuelossesdata_tmp); //  
		valuelossesdata_tmp*=100000; //    
		}
	else valuelossesdata_tmp=0; //   
	return (dword)valuelossesdata_tmp; //  
	}

floatpoint CAP_CorrectLosses(floatpoint valuelossesdata_prm) //  
	{
	floatpoint correctconstants_tmp=0; // 
	switch(alter_cap.measuredatafrequency)
		{
		case 10000 ... 17153: correctconstants_tmp=CAP_CorrectFactor(1,0,valuelossesdata_prm); //  
							break;
		case 17154 ... 21159: correctconstants_tmp=CAP_CorrectFactor(2,1,valuelossesdata_prm); //  
							break;
		case 21160 ... 29732: correctconstants_tmp=CAP_CorrectFactor(3,2,valuelossesdata_prm); //  
							break;
		case 29733 ... 44548: correctconstants_tmp=CAP_CorrectFactor(4,3,valuelossesdata_prm); //  
							break;
		case 44549 ... 54150: correctconstants_tmp=CAP_CorrectFactor(5,4,valuelossesdata_prm); //  
							break;
		case 54151 ... 61169: correctconstants_tmp=CAP_CorrectFactor(6,5,valuelossesdata_prm); //  
							break;
		case 61170 ... 79597: correctconstants_tmp=CAP_CorrectFactor(7,6,valuelossesdata_prm); //  
							break;
		case 79598 ... 101126: correctconstants_tmp=CAP_CorrectFactor(8,7,valuelossesdata_prm); //  
							break;
		case 101127 ... 108598: correctconstants_tmp=CAP_CorrectFactor(9,8,valuelossesdata_prm); //  
							break;
		case 108599 ... 118275: correctconstants_tmp=CAP_CorrectFactor(10,9,valuelossesdata_prm); //  
							break;
		case 118276 ... 129469: correctconstants_tmp=CAP_CorrectFactor(11,10,valuelossesdata_prm); //  
							break;
		case 129470 ... 136401: correctconstants_tmp=CAP_CorrectFactor(12,11,valuelossesdata_prm); //  
							break;
		case 136402 ... 137439: correctconstants_tmp=CAP_CorrectFactor(13,12,valuelossesdata_prm); //  
							break;
		case 137440 ... 139851: correctconstants_tmp=CAP_CorrectFactor(14,13,valuelossesdata_prm); //  
							break;
		case 139852 ... 142275: correctconstants_tmp=CAP_CorrectFactor(15,14,valuelossesdata_prm); //  
							break;
		case 142276 ... 142384: correctconstants_tmp=CAP_CorrectFactor(16,15,valuelossesdata_prm); //  
							break;
		case 142385 ... 142646: correctconstants_tmp=CAP_CorrectFactor(17,16,valuelossesdata_prm); //  
							break;
		case 142647 ... 142949: correctconstants_tmp=CAP_CorrectFactor(18,17,valuelossesdata_prm); //  
							break;
		case 142950 ... 143100: correctconstants_tmp=CAP_CorrectFactor(19,18,valuelossesdata_prm); //  
		}
	return correctconstants_tmp; //   
	}

floatpoint CAP_CorrectFactor(byte secondarynumberdata_prm,byte primarynumberdata_prm,floatpoint valuelossesdata_prm) //   
	{
	dword frequencyrange_tmp=0; // 
	floatpoint correctconstants_tmp=0;
	floatpoint intermediatedata_tmp=0;
	intermediatedata_tmp=pgm_read_float(&Cap_CorrectConstants[secondarynumberdata_prm])-pgm_read_float(&Cap_CorrectConstants[primarynumberdata_prm]); //   
	frequencyrange_tmp=pgm_read_dword(&Cap_FrequencyRange[secondarynumberdata_prm])-pgm_read_dword(&Cap_FrequencyRange[primarynumberdata_prm]); //  
	correctconstants_tmp=intermediatedata_tmp/(floatpoint)frequencyrange_tmp; //    
	intermediatedata_tmp=((floatpoint)alter_cap.measuredatafrequency-(floatpoint)pgm_read_dword(&Cap_FrequencyRange[primarynumberdata_prm]))*correctconstants_tmp; //    
	correctconstants_tmp=valuelossesdata_prm/(intermediatedata_tmp+pgm_read_float(&Cap_CorrectConstants[primarynumberdata_prm])); //  
	return correctconstants_tmp; //   
	}

dword CAP_QualityCapacity(void) //   
	{
	dword qualitycapacity_tmp=0; // 
	qualitycapacity_tmp=1000000/alter_cap.datalossescapacity; //   
	return qualitycapacity_tmp; //  
	}

dword CAP_CalculationCapextra(void) //    
	{
	dword inductancedata_tmp=0; // 
	ldword extracapacity_tmp=0;
	ldword totalcapacity_tmp=0;
	floatpoint intermediatedata_tmp=0;
	inductancedata_tmp=eeprom_read_dword(&Cap_SupportInductance); //   
	intermediatedata_tmp=2*M_PI*(floatpoint)alter_cap.currentdatafrequency; //  
	intermediatedata_tmp*=intermediatedata_tmp; //  
	intermediatedata_tmp/=1000; //  
	intermediatedata_tmp*=(floatpoint)inductancedata_tmp; // 
	totalcapacity_tmp=1000000000000000000/(ldword)intermediatedata_tmp; //  
	extracapacity_tmp=(ldword)alter_cap.supportcapacity*totalcapacity_tmp/((ldword)alter_cap.supportcapacity-totalcapacity_tmp); //  
	return (dword)extracapacity_tmp; //  
	}

dword CAP_CalculationInductance(void) //    
	{
	ldword inductancedata_tmp=0; // 
	floatpoint intermediatedata_tmp=0;
	intermediatedata_tmp=2*M_PI*(floatpoint)alter_cap.currentdatafrequency; //  
	intermediatedata_tmp*=intermediatedata_tmp; //  
	intermediatedata_tmp/=1000; //  
	intermediatedata_tmp*=(floatpoint)alter_cap.supportcapacity; // 
	inductancedata_tmp=1000000000000000000/(ldword)intermediatedata_tmp; //  
	return (dword)inductancedata_tmp; //  
	}

void CAP_ControlMode(byte readoutcode_prm) //   
	{
	switch(readoutcode_prm) //   
		{
		case 1: switch(alter_cap.countlineposition) //       Left
					{	
					case 0: if(alter_cap.countspotposition==0) alter_cap.countspotposition=1; //           
							else alter_cap.countspotposition=0; //    
							break;
					case 1: asm("nop"); // 
							break;
					case 2: if(alter_cap.countspotposition==1) alter_cap.countspotposition=2; //  2   1     
							else alter_cap.countspotposition=1; //  1   2
							break;
					case 3: if(alter_cap.countspotposition==1) alter_cap.countspotposition=2; //  2   1     
							else alter_cap.countspotposition=1; //  1   2
							break;
					case 4: if(alter_cap.countspotposition>1) alter_cap.countspotposition--; //            1
							else alter_cap.countspotposition=4; //  4     1
							break;
					case 5: if(alter_cap.countspotposition==1) alter_cap.countspotposition=2; //  2   1       
							else alter_cap.countspotposition=1; //  1   2
					}
				break;
		case 2: switch(alter_cap.countlineposition) //       Rigt
					{
					case 0: if(alter_cap.countspotposition==0) alter_cap.countspotposition=1; //      
							else alter_cap.countspotposition=0; //   
							break;
					case 1: asm("nop"); // 
							break;
					case 2: if(alter_cap.countspotposition==1) alter_cap.countspotposition=2; //  2   1     
							else alter_cap.countspotposition=1; //  1   2
							break;
					case 3: if(alter_cap.countspotposition==1) alter_cap.countspotposition=2; //  2   1     
							else alter_cap.countspotposition=1; //  1   2
							break;
					case 4: alter_cap.countspotposition++; //         
							if(alter_cap.countspotposition>4) alter_cap.countspotposition=1; //  1     4
							break;
					case 5: if(alter_cap.countspotposition==1) alter_cap.countspotposition=2; //  2   1       
							else alter_cap.countspotposition=1; //  1   2
					}
				break;
		case 3: if(alter_cap.countspotposition==1) //  
					{
					if(alter_cap.countlineaddition==0) //   
						{
						alter_cap.countlineposition++; //  
						if(alter_cap.countlineposition>5) alter_cap.countlineposition=0; //    5
						CAP_UpdateLine(); // 
						}
					else asm("nop"); //       
					}
				else CAP_CorrectCalibrate(FALSE); //      
				break;
		case 4: if(alter_cap.countspotposition==1) //  
					{
					if(alter_cap.countlineaddition==0) //   
						{
						if(alter_cap.countlineposition>0) alter_cap.countlineposition--; //     
						else alter_cap.countlineposition=5; //   5  
						CAP_UpdateLine(); // 
						}
					else asm("nop"); //       
					}
				else CAP_CorrectCalibrate(TRUE); //      
				break;
		case 5: switch(alter_cap.countlineposition) //   
					{
					case 0: if(indx_cap.capacityreactive==FALSE) indx_cap.capacityreactive=TRUE; //    
							else indx_cap.capacityreactive=FALSE; //    
							indx_cap.initializecapmode=TRUE; //      
							break;
					case 1: alter_cap.countlineaddition++; //   
							if(alter_cap.countlineaddition>3) alter_cap.countlineaddition=0; //     
							switch(alter_cap.countlineaddition) //  
								{
								case 0: SYM_OutcharLine(0,5,1); // : F=       
										break;
								case 1: SYM_OutcharLine(0,6,1); // : T=  ,    
										break;
								case 2: SYM_OutcharLine(2,8,1); // : tg&= ,
										break;
								case 3: SYM_OutcharLine(2,9,1); // : Q=     ,
								}
							break;
					case 2: if((indx_cap.writeoffsetcapacity==FALSE)&&(alter_cap.countspotposition==2)) indx_cap.writeoffsetcapacity=TRUE; //     
							else asm("nop"); // 
							break;
					case 3: if(alter_cap.countspotposition==2) CAP_SavingInductance(); //        
							else asm("nop"); // 
							break;
					case 4: asm("nop"); // 
							break;
					case 5: if(alter_cap.countspotposition==2) CAP_SavingextraCapacity(); //        
							else asm("nop"); // 
					}
		}
	indx_cap.displaydatameasure=TRUE; //   
	switch(alter_cap.countlineposition) //  
		{
		case 0: alter_cap.linepositioncursor=0; //   
				switch(alter_cap.countspotposition) //   
					{
					case 0: alter_cap.spotpositioncursor=0; //        
							break;
					case 1: alter_cap.spotpositioncursor=5; //         /
					}
				break;
		case 1: alter_cap.linepositioncursor=1; //   
				alter_cap.spotpositioncursor=5; //       /
				break;
		case 2: alter_cap.linepositioncursor=1; //   
				switch(alter_cap.countspotposition) //   
					{
					case 1: alter_cap.spotpositioncursor=9; //      
							break;
					case 2: alter_cap.spotpositioncursor=12; //      
					}
				break;
		case 3: alter_cap.linepositioncursor=1; //   
				switch(alter_cap.countspotposition) //   
					{
					case 1: alter_cap.spotpositioncursor=9; //      
							break;
					case 2: alter_cap.spotpositioncursor=12; //      
					}
				break;
		case 4: alter_cap.linepositioncursor=1; //   
				switch(alter_cap.countspotposition) //       
					{
					case 1: alter_cap.spotpositioncursor=9; //       
							break;
					case 2: alter_cap.spotpositioncursor=13; //    
							break;
					case 3: alter_cap.spotpositioncursor=14; //    
							break;
					case 4: alter_cap.spotpositioncursor=16; //    
					}
				break;
		case 5: alter_cap.linepositioncursor=1; //   
				switch(alter_cap.countspotposition) //      
					{
					case 1: alter_cap.spotpositioncursor=9; //      
							break;
					case 2: alter_cap.spotpositioncursor=12; //     
					}
		}
	}

void CAP_UpdateLine(void) //   
	{
	if((alter_cap.countlineposition!=5)&&(indx_cap.inclusimultiplifrequ==TRUE)) //       
		{
		indx_cap.inclusimultiplifrequ=FALSE; // 
		CON_SwitchDevice(FALSE,8); //  
		}
	switch(alter_cap.countlineposition) // 
		{
		case 0 ... 1: SYM_OutcharLine(0,5,1); // : F=       
				break;
		case 2: SYM_OutcharLine(3,10,1); // : .C0      ,  
				break;
		case 3: SYM_OutcharLine(3,14,1); // : .L     ,   
				break;
		case 4: SYM_OutcharLine(3,11,1); // : .Cp    ,  
				break;
		case 5: SYM_OutcharLine(3,12,1); // : .Ce     ,  
				if(indx_cap.inclusimultiplifrequ==FALSE) //  
					{
					indx_cap.inclusimultiplifrequ=TRUE; //   
					CON_SwitchDevice(TRUE,8); //  
					}
		}
	}

void CAP_CorrectCalibrate(bit indxincdecdata_prm) //   
	{
	switch(alter_cap.countlineposition)
		{
		case 0 ... 3: asm("nop"); // 
				break;
		case 4: CAP_CorrectProbe(indxincdecdata_prm); //    
				break;
		case 5: asm("nop"); // 
		}
	}

void CAP_CorrectProbe(bit indxincdecdata_prm) //     
	{
	dword capacitydata_tmp=0; // 
	capacitydata_tmp=eeprom_read_dword(&Cap_ProbeCapacity); //    
	switch(alter_cap.countspotposition) //   
		{
		case 1: asm("nop"); // 
				break;
		case 2: if(indxincdecdata_prm==FALSE) capacitydata_tmp+=100; //   100   
				else if((indxincdecdata_prm==TRUE)&&(capacitydata_tmp>100)) capacitydata_tmp-=100; //   100   
				break;
		case 3: if(indxincdecdata_prm==FALSE) capacitydata_tmp+=10; //   10   
				else if((indxincdecdata_prm==TRUE)&&(capacitydata_tmp>10)) capacitydata_tmp-=10; //   10   
				break;
		case 4: if(indxincdecdata_prm==FALSE) capacitydata_tmp+=1; //   1   
				else if((indxincdecdata_prm==TRUE)&&(capacitydata_tmp>1)) capacitydata_tmp-=1; //   1   
		}
	eeprom_update_dword(&Cap_ProbeCapacity,capacitydata_tmp); //    
	}

void CAP_SavingextraCapacity(void) //    
	{
	dword extracapacity_tmp=0; // 
	extracapacity_tmp=CAP_CalculationCapextra(); //     
	eeprom_update_dword(&Cap_ExtraCapacity,extracapacity_tmp); //  
	}

void CAP_SavingInductance(void) //    
	{
	dword inductancedata_tmp=0; // 
	inductancedata_tmp=CAP_CalculationInductance(); //     
	eeprom_update_dword(&Cap_SupportInductance,inductancedata_tmp); //  
	CAP_ConstantaMeasure(); // 
	}

dword CAP_ReadsupportInductance(void) //    
	{
	dword inductancedata_tmp=0; // 
	inductancedata_tmp=eeprom_read_dword(&Cap_SupportInductance); //   
	return inductancedata_tmp; //  
	}














