/*************************************
* Esrmeasure.c
*     ESR
* : 19.02.2021 10:45:14
* :  .
*************************************/

// 
#include <avr/io.h>
#include <avr/eeprom.h>
#include <util/delay_basic.h>
#include "Main.h"
#include "Esrmeasure.h"
#include "Othercalibrations.h"
#include "Adconverter.h"
#include "Connectmode.h"
#include "Systemcontrol.h"
#include "Cappmeasure.h"

EEMEM static word Esr_Current3maValue=ESRCURRENT_VALUE3MA; //    3 
EEMEM static word Esr_Current10maValue=ESRCURRENT_VALUE10MA; //    10 
EEMEM static word Esr_Current50maValue=ESRCURRENT_VALUE50MA; //    50 

var_esr alter_esr={0}; //  

void ESR_ReadThreshold(void) //     
	{
	bit outcicleindx_tmp=FALSE; // 
	ldword adcvaluedata_tmp=0;
	while(outcicleindx_tmp==FALSE) //   
		{
		adcvaluedata_tmp=(ldword)ADC_AverageAsymmdata(FALSE,10,ADCASYMM_3INPUT); //  
		if(adcvaluedata_tmp==alter_esr.voltagethreshold) outcicleindx_tmp=TRUE; //       
		else //      
			{
			alter_esr.voltagethreshold=adcvaluedata_tmp; //  
			SYS_TimeWaiting(100); // 100 
			}
		}
	alter_esr.voltagethreshold=adcvaluedata_tmp*ESRTHRESHOLD_RESISTANCE*1000000/ESRTEST_RESISTANCE; //  
	}

word ESR_ReadCurrent(byte numbercurrentdata_prm) //   
	{
	word currentvalue_tmp=0; // 
	switch(numbercurrentdata_prm) //  
		{
		case 0: currentvalue_tmp=eeprom_read_word(&Esr_Current3maValue); //    3 
				break;
		case 1: currentvalue_tmp=eeprom_read_word(&Esr_Current10maValue); //    10 
				break;
		case 2: currentvalue_tmp=eeprom_read_word(&Esr_Current50maValue); //    50 
		}
	return currentvalue_tmp; //  
	}

void ESR_WriteCurrent(word currentvalue_prm,byte numbercurrentdata_prm) //   
	{
	switch(numbercurrentdata_prm) //  
		{
		case 0: eeprom_update_word(&Esr_Current3maValue,currentvalue_prm); //    3 
				break;
		case 1: eeprom_update_word(&Esr_Current10maValue,currentvalue_prm); //    10 
				break;
		case 2: eeprom_update_word(&Esr_Current50maValue,currentvalue_prm); //    50 
		}
	}

void ESR_TimeDischarge(dword timedischargecapacity_prm) //    
	{
	dword countdischargetime_tmp; // 
	countdischargetime_tmp=timedischargecapacity_prm/4; //   
	while(countdischargetime_tmp>0) //   
		{
		_delay_loop_2(4); //  1    
		countdischargetime_tmp--; //  
		}
	}

dword ESR_EsrdataCapacity(ldword capacitancedata_prm,dword timedischargecapacity_prm) //   ESR 
	{
	dword eqseriesresistance_tmp=0; // 
	CAPP_InitializeVariable(); // 
	CON_SwitchDevice(FALSE,0); //    
	CON_SwitchDevice(TRUE,4); //   ESR
	CON_SwitchDevice(TRUE,5); //    
	ESR_TimeDischarge(timedischargecapacity_prm); //     
	CON_SwitchDevice(TRUE,6); //    
	CON_SwitchDevice(FALSE,5); //    
	eqseriesresistance_tmp=CAPP_CycleEsrmeasure(capacitancedata_prm); //  ESR
	CON_SwitchDevice(FALSE,0); //    
	CON_SwitchDevice(TRUE,5); //    
	CON_SettingCurrent(1); //  
	CON_SwitchDevice(TRUE,3); //   
	return eqseriesresistance_tmp; //  ESR 
	}

dword ESR_ProcessMeasure(byte currentindx_prm,dword primintervaltime_prm,dword secintervaltime_prm) //    ESR 
	{
	word currentvalue_tmp=0; // 
	ldword primintervaltime_tmp=0;
	ldword secintervaltime_tmp=0;
	ldword intervaltimedata_tmp=0;
	ldword esrcapacitancedata_tmp=0;
	primintervaltime_tmp=(ldword)primintervaltime_prm*ESRCPU_PERIOD; //   
	secintervaltime_tmp=(ldword)secintervaltime_prm*ESRCPU_PERIOD; //   
	if(secintervaltime_tmp>(primintervaltime_tmp*2)) //    
		{
		intervaltimedata_tmp=alter_esr.voltagethreshold*(secintervaltime_tmp-(primintervaltime_tmp*2)); //  
		switch(currentindx_prm) //  
			{
			case 1: currentvalue_tmp=eeprom_read_word(&Esr_Current3maValue); //   3 
					break;
			case 2: currentvalue_tmp=eeprom_read_word(&Esr_Current10maValue); //   10 
					break;
			case 3: currentvalue_tmp=eeprom_read_word(&Esr_Current50maValue); //   50 
			}
		esrcapacitancedata_tmp=intervaltimedata_tmp/((ldword)currentvalue_tmp*(secintervaltime_tmp-primintervaltime_tmp)); //  ESR
		}
	else esrcapacitancedata_tmp=0; //   ESR
	return (dword)esrcapacitancedata_tmp; //  ESR
	}


















