/*****************************************
* Othercalibrations.c
*    
* : 19.02.2021 21:57:10
* :  .
******************************************/

// 
#include <avr/io.h>
#include <avr/eeprom.h>
#include "Othercalibrations.h"
#include "Main.h"
#include "Systemcontrol.h"
#include "Outmessage.h"
#include "Adconverter.h"
#include "Connectmode.h"
#include "Esrmeasure.h"
#include "Symbolcharge.h"

EEMEM static word Oth_CalibrateRes12_18om=OTHRES12_18OM; //      3   50 
EEMEM static word Oth_CalibrateRes51_68om=OTHRES51_68OM; //      10 

bit_oth indx_oth={0}; // 
var_oth alter_oth={0,0,0,0}; // 

void OTH_InitializeCalibrate(byte codecalibrate_prm) //   
	{
	bit indxonoffaudio_tmp=FALSE; // 
	if(SYS_ReadindxOnoffaudio()>0) indxonoffaudio_tmp=TRUE; //      
	switch(codecalibrate_prm) //  
		{
		case 4: CON_RefresCalibrate(); //    
				OUT_ArefdataTitle(); //    
				break;
		case 5: CON_RefresCalibrate(); //    
				OUT_VddrefdataTitle(); //     Vdd
				break;
		case 6: CON_FrequpiezoCalibrate(); //      
				OUT_FrequpiezoTitle(indxonoffaudio_tmp); //    
				break;
		case 7: CON_RefresCalibrate(); //    
				OUT_Res12_18Title(); //    12-18 
				break;
		case 8: CON_RefresCalibrate(); //    
				OUT_Res51_68Title(); //    51-68 
				break;
		case 9: SYM_CalibrGraphics(); //        3 
				CON_Curr3maCalibrate(); //   
				OUT_Current3maTitle(); //     3 
				break;
		case 10: SYM_CalibrGraphics(); //        10 
				CON_Curr10maCalibrate(); //   
				OUT_Current10maTitle(); //     10 
				break;
		case 11: SYM_CalibrGraphics(); //        50 
				CON_Curr50maCalibrate(); //   
				OUT_Current50maTitle(); //     50 
		}
	}

byte OTH_ReadCursorposition(bit indxplaceposition_prm) //       
	{
	byte cursorposition_tmp=0; // 
	switch(indxplaceposition_prm) // /
		{
		case FALSE: cursorposition_tmp=alter_oth.spotpositioncursor; //    
					break;
		case TRUE: cursorposition_tmp=alter_oth.linepositioncursor; //   
		}
	return cursorposition_tmp; //     
	}

void OTH_ViewCalibratevalue(byte datacodemode_prm) //   
	{
	switch(datacodemode_prm) //  
		{
		case 4:	OTH_ViewArefvoltage(); //    
				break;
		case 5: OTH_ViewVddrefvoltage(); //     Vdd
				break;
		case 6: OTH_ViewFrequencypiezo(); //    
				break;
		case 7: OTH_ViewResistance(FALSE); //   12-18 
				break;
		case 8: OTH_ViewResistance(TRUE); //   51-68 
				break;
		case 9: OTH_ViewPwmcurrent(0); //     3 
				break;
		case 10: OTH_ViewPwmcurrent(1); //     10 
				break;
		case 11: OTH_ViewPwmcurrent(2); //     50 
		}
	}

void OTH_ViewArefvoltage(void) //    
	{
	word arefvoltage_tmp=0; // 
	word checkvoltage_tmp=0;
	checkvoltage_tmp=ADC_AverageAsymmdata(FALSE,10,ADCASYMM_3INPUT); //  
	arefvoltage_tmp=ADC_ReadRefvoltage(FALSE); //  
	OUT_DataRefcalibrate(checkvoltage_tmp,arefvoltage_tmp); //        
	}

void OTH_ViewVddrefvoltage(void) //     Vdd
	{
	word vddrefvoltage_tmp=0; // 
	word checkvoltage_tmp=0;
	checkvoltage_tmp=ADC_AverageAsymmdata(TRUE,10,ADCASYMM_3INPUT); //  
	vddrefvoltage_tmp=ADC_ReadRefvoltage(TRUE); //  
	OUT_DataRefcalibrate(checkvoltage_tmp,vddrefvoltage_tmp); //         
	}

void OTH_ControlModecalibrate(byte numdatamode_prm,byte readoutcode_prm) //   
	{
	switch(numdatamode_prm) //  
		{
		case 4:	OTH_ControlArefvoltage(readoutcode_prm); //    
				break;
		case 5: OTH_ControlVddrefvoltage(readoutcode_prm); //     Vdd
				break;
		case 6: OTH_ControlFrequpiezo(readoutcode_prm); //     
				break;
		case 7: OTH_ControlRescalibrate(FALSE,readoutcode_prm); //    12-18 
				break;
		case 8: OTH_ControlRescalibrate(TRUE,readoutcode_prm); //    51-68 
				break;
		case 9: OTH_ControlPwmcalibrate(1,readoutcode_prm); //    3 
				break;
		case 10: OTH_ControlPwmcalibrate(2,readoutcode_prm); //    10 
				break;
		case 11: OTH_ControlPwmcalibrate(3,readoutcode_prm); //    50 
		}
	}

void OTH_ControlArefvoltage(byte readoutcode_prm) //    
	{
	switch(readoutcode_prm) //  
		{
		case 1: if(alter_oth.countspotposition>0) alter_oth.countspotposition--; //         Left
				else alter_oth.countspotposition=3; //    
				break;
		case 2: alter_oth.countspotposition++; //      Rigt
				if(alter_oth.countspotposition>3) alter_oth.countspotposition=0; //     
				break;
		case 3: OTH_CorrectArefvoltage(FALSE); //      Up
				break;
		case 4: OTH_CorrectArefvoltage(TRUE); //      Down
				break;
		case 5: asm("nop"); //     Enter
		}
	switch(alter_oth.countspotposition) //   
		{
		case 0: alter_oth.spotpositioncursor=0; alter_oth.linepositioncursor=0; //   
				break;
		case 1: alter_oth.spotpositioncursor=15; alter_oth.linepositioncursor=1; //     
				break;
		case 2: alter_oth.spotpositioncursor=16; alter_oth.linepositioncursor=1; //     
				break;
		case 3: alter_oth.spotpositioncursor=17; alter_oth.linepositioncursor=1; //     
		}
	}

void OTH_CorrectArefvoltage(bit indxincdecdata_prm) //    
	{
	dword calibratedata_tmp=0; // 
	calibratedata_tmp=ADC_ReadRefvoltage(FALSE); //   
	switch(alter_oth.countspotposition) //  
		{
		case 0: asm("nop"); // 
				break;
		case 1: if(indxincdecdata_prm==FALSE) calibratedata_tmp+=100; //   100   
				else if((indxincdecdata_prm==TRUE)&&(calibratedata_tmp>100)) calibratedata_tmp-=100; //   100   
				break;
		case 2: if(indxincdecdata_prm==FALSE) calibratedata_tmp+=10; //   10   
				else if((indxincdecdata_prm==TRUE)&&(calibratedata_tmp>10)) calibratedata_tmp-=10; //   10   
				break;
		case 3: if(indxincdecdata_prm==FALSE) calibratedata_tmp+=1; //   1   
				else if((indxincdecdata_prm==TRUE)&&(calibratedata_tmp>1)) calibratedata_tmp-=1; //   1   
		}
	ADC_WriteRefvoltage(FALSE,calibratedata_tmp); //   
	}

void OTH_ControlVddrefvoltage(byte readoutcode_prm) //     Vdd
	{
	switch(readoutcode_prm) //  
		{
		case 1: if(alter_oth.countspotposition>0) alter_oth.countspotposition--; //         Left
				else alter_oth.countspotposition=2; //    
				break;
		case 2: alter_oth.countspotposition++; //      Down
				if(alter_oth.countspotposition>2) alter_oth.countspotposition=0; //     
				break;
		case 3: OTH_CorrectVddrefvoltage(FALSE); //       Up
				break;
		case 4: OTH_CorrectVddrefvoltage(TRUE); //       Down
				break;
		case 5: asm("nop"); //     Enter
		}
	switch(alter_oth.countspotposition) //   
		{
		case 0: alter_oth.spotpositioncursor=0; alter_oth.linepositioncursor=0; //   
				break;
		case 1: alter_oth.spotpositioncursor=16; alter_oth.linepositioncursor=1; //     
				break;
		case 2: alter_oth.spotpositioncursor=17; alter_oth.linepositioncursor=1; //     
		}
	}

void OTH_CorrectVddrefvoltage(bit indxincdecdata_prm) //    
	{
	dword calibratedata_tmp=0; // 
	calibratedata_tmp=ADC_ReadRefvoltage(TRUE); //   
	switch(alter_oth.countspotposition) //  
		{
		case 0: asm("nop"); // 
				break;
		case 1: if(indxincdecdata_prm==FALSE) calibratedata_tmp+=10; //   10   
				else if((indxincdecdata_prm==TRUE)&&(calibratedata_tmp>10)) calibratedata_tmp-=10; //   10   
				break;
		case 2: if(indxincdecdata_prm==FALSE) calibratedata_tmp+=1; //   1   
				else if((indxincdecdata_prm==TRUE)&&(calibratedata_tmp>1)) calibratedata_tmp-=1; //   1   
		}
	ADC_WriteRefvoltage(TRUE,calibratedata_tmp); //   
	}

void OTH_ViewFrequencypiezo(void) //    
	{
	dword datafrequencypiezo_tmp=0; // 
	datafrequencypiezo_tmp=SYS_ReadFrequencypiezo(); //   
	OUT_DataFrequencypiezo(datafrequencypiezo_tmp); //     
	}

void OTH_ControlFrequpiezo(byte readoutcode_prm) //     
	{
	switch(readoutcode_prm) //   
		{
		case 1: switch(alter_oth.countlineposition) //     Left
					{
					case 0: if(alter_oth.countspotposition==0) alter_oth.countspotposition=1; //          
							else alter_oth.countspotposition=0; //      
							break;
					case 1: if(alter_oth.countspotposition>0) alter_oth.countspotposition--; //        
							else alter_oth.countspotposition=4; //    
					}
				break;
		case 2: switch(alter_oth.countlineposition) //     Rigt
					{
					case 0: if(alter_oth.countspotposition==0) alter_oth.countspotposition=1; //         
							else alter_oth.countspotposition=0; //      
							break;
					case 1: alter_oth.countspotposition++; //     
							if(alter_oth.countspotposition>4) alter_oth.countspotposition=0; //     
					}
				break;
		case 3: switch(alter_oth.countlineposition) //     Up
					{
					case 0: if(alter_oth.countlineposition==0) alter_oth.countlineposition=1; //       
							else alter_oth.countlineposition=0; //    
							alter_oth.countspotposition=0; //    
							break;
					case 1: OTH_CorrectFrequpiezo(FALSE); //   
					}
				break;
		case 4: switch(alter_oth.countlineposition) //     Down
					{
					case 0: if(alter_oth.countlineposition==0) alter_oth.countlineposition=1; //       
							else alter_oth.countlineposition=0; //    
							alter_oth.countspotposition=0; //    
							break;
					case 1: OTH_CorrectFrequpiezo(TRUE); //   
					}
				break;
		case 5: switch(alter_oth.countlineposition) //     Enter
					{
					case 0: if(SYS_ReadindxOnoffaudio()>0) //   
								{
								SYS_WriteindxOnoffaudio(0); //  
								OUT_FrequpiezoTitle(FALSE); // 
								}
							else //   
								{
								SYS_WriteindxOnoffaudio(1); //  
								OUT_FrequpiezoTitle(TRUE); // 
								}
							break;
					case 1: asm("nop"); // 
					}
		}
	switch(alter_oth.countlineposition) //  
		{
		case 0: alter_oth.linepositioncursor=0; //  
				switch(alter_oth.countspotposition) //   
					{
					case 0: alter_oth.spotpositioncursor=0;  //   
							break;
					case 1: alter_oth.spotpositioncursor=13; //   /   
					}
				break;
		case 1: alter_oth.linepositioncursor=1; //  
				switch(alter_oth.countspotposition) //   
					{
					case 0: alter_oth.spotpositioncursor=9;  //     
							break;
					case 1: alter_oth.spotpositioncursor=13; //      
							break;
					case 2: alter_oth.spotpositioncursor=14; //      
							break;
					case 3: alter_oth.spotpositioncursor=15; //      
							break;
					case 4: alter_oth.spotpositioncursor=16; //      
					}
		}
	}

void OTH_CorrectFrequpiezo(bit indxincdecdata_prm) //    
	{
	dword calibratedata_tmp=0; // 
	calibratedata_tmp=SYS_ReadFrequencypiezo(); //   
	switch(alter_oth.countspotposition) //  
		{
		case 0: if(alter_oth.countlineposition==0) alter_oth.countlineposition=1; //       
				else
					{
					alter_oth.countlineposition=0; //    
					alter_oth.countspotposition=1; //     
					}
				break;
		case 1: if(indxincdecdata_prm==FALSE) calibratedata_tmp+=1000; //   1000   
				else if((indxincdecdata_prm==TRUE)&&(calibratedata_tmp>1000)) calibratedata_tmp-=1000; //   1000   
				break;
		case 2: if(indxincdecdata_prm==FALSE) calibratedata_tmp+=100; //   100   
				else if((indxincdecdata_prm==TRUE)&&(calibratedata_tmp>100)) calibratedata_tmp-=100; //   100   
				break;
		case 3: if(indxincdecdata_prm==FALSE) calibratedata_tmp+=10; //   10   
				else if((indxincdecdata_prm==TRUE)&&(calibratedata_tmp>10)) calibratedata_tmp-=10; //   10   
				break;
		case 4: if(indxincdecdata_prm==FALSE) calibratedata_tmp++; //   1   
				else if((indxincdecdata_prm==TRUE)&&(calibratedata_tmp>1)) calibratedata_tmp--; //   1   
		}
	SYS_WriteFrequencypiezo(calibratedata_tmp); //    
	}

void OTH_ViewResistance(bit indxresistance_prm) //   
	{
	word dataresistance_tmp=0; // 
	switch(indxresistance_prm) //  
		{
		case FALSE: dataresistance_tmp=eeprom_read_word(&Oth_CalibrateRes12_18om); //   12  18 
					break;
		case TRUE: dataresistance_tmp=eeprom_read_word(&Oth_CalibrateRes51_68om); //   51  68 
		}
	OUT_DataRescalibrate(dataresistance_tmp); //    
	}

void OTH_ControlRescalibrate(bit indxresistance_prm,byte readoutcode_prm) //    
	{
	switch(readoutcode_prm) //  
		{
		case 1: if(alter_oth.countspotposition>0) alter_oth.countspotposition--; //         Left
				else alter_oth.countspotposition=3; //    
				break; 
		case 2: alter_oth.countspotposition++; //      Rigt
				if(alter_oth.countspotposition>3) alter_oth.countspotposition=0; //     
				break; 
		case 3: if(indxresistance_prm==FALSE) OTH_CorrectResistance(FALSE,FALSE); //   12-18     Up
				else OTH_CorrectResistance(TRUE,FALSE); //   51-68 
				break; 
		case 4: if(indxresistance_prm==FALSE) OTH_CorrectResistance(FALSE,TRUE); //   12-18     Down
				else OTH_CorrectResistance(TRUE,TRUE); //   51-68 
				break; 
		case 5: asm("nop"); //     Enter
		}
	switch(alter_oth.countspotposition) //   
		{
		case 0: alter_oth.spotpositioncursor=0; alter_oth.linepositioncursor=0; //   
				break;
		case 1: alter_oth.spotpositioncursor=8; alter_oth.linepositioncursor=0; //     
				break;
		case 2: alter_oth.spotpositioncursor=9; alter_oth.linepositioncursor=0; //     
				break;
		case 3: alter_oth.spotpositioncursor=11; alter_oth.linepositioncursor=0; //     
		}
	}

void OTH_CorrectResistance(bit indxresistance_prm,bit indxincdecdata_prm) //   
	{
	word calibratedata_tmp=0; // 
	switch(indxresistance_prm) //  
		{
		case FALSE: calibratedata_tmp=eeprom_read_word(&Oth_CalibrateRes12_18om); //   12-18 
					break;
		case TRUE: calibratedata_tmp=eeprom_read_word(&Oth_CalibrateRes51_68om); //   51-68 
		}
	switch(alter_oth.countspotposition) //  
		{
		case 0: asm("nop"); // 
				break;
		case 1: if(indxincdecdata_prm==FALSE) calibratedata_tmp+=100; //   100   
				else if((indxincdecdata_prm==TRUE)&&(calibratedata_tmp>100)) calibratedata_tmp-=100; //   100   
				break;
		case 2: if(indxincdecdata_prm==FALSE) calibratedata_tmp+=10; //   10   
				else if((indxincdecdata_prm==TRUE)&&(calibratedata_tmp>10)) calibratedata_tmp-=10; //   10   
				break;
		case 3: if(indxincdecdata_prm==FALSE) calibratedata_tmp+=1; //   1   
				else if((indxincdecdata_prm==TRUE)&&(calibratedata_tmp>1)) calibratedata_tmp-=1; //   1   
		}
	switch(indxresistance_prm) //   
		{
		case FALSE: eeprom_update_word(&Oth_CalibrateRes12_18om,calibratedata_tmp); //   12-18 
					break;
		case TRUE: eeprom_update_word(&Oth_CalibrateRes51_68om,calibratedata_tmp); //   51-68 
		}
	}

word OTH_ReadResistancedata(bit indxresistance_prm) //    
	{
	word resistancedata_tmp=0; // 
	switch(indxresistance_prm) //  
		{
		case FALSE: resistancedata_tmp=eeprom_read_word(&Oth_CalibrateRes12_18om); //  12-18 
					break;
		case TRUE: resistancedata_tmp=eeprom_read_word(&Oth_CalibrateRes51_68om); //  51-68 
		}
	return resistancedata_tmp; //   
	}

void OTH_ViewPwmcurrent(byte numberdata_prm) //        
	{
	word actualpwmdata_tmp=0; // 
	dword actualcurrdata_tmp=0;
	word actualresdata_tmp=0;
	word actualvoltage_tmp=0;
	switch(numberdata_prm) //   
		{
		case 0: actualresdata_tmp=OTH_ReadResistancedata(FALSE); //   12-18 
				actualpwmdata_tmp=CON_ReadPwmdata(1); //      3 
				actualvoltage_tmp=ADC_AverageSymmdata(FALSE,10,ADCSYMM10X_10INPUT); //    
				if(actualvoltage_tmp<1000) //   
					{
					actualcurrdata_tmp=(dword)actualvoltage_tmp*1000/actualresdata_tmp; //  
					if(indx_oth.writecurrentdata==TRUE) //     
						{
						ESR_WriteCurrent((word)actualcurrdata_tmp,0); //    eeprom
						indx_oth.writecurrentdata=FALSE; // 
						}
					if(alter_oth.countspotposition==3) actualcurrdata_tmp=(dword)ESR_ReadCurrent(0); //    eeprom     
					}
				else actualcurrdata_tmp=55555; //     55555    
				break;
		case 1: actualresdata_tmp=OTH_ReadResistancedata(TRUE); //   51-68 
				actualpwmdata_tmp=CON_ReadPwmdata(2); //      10 
				actualvoltage_tmp=ADC_AverageSymmdata(FALSE,10,ADCSYMM1X_12INPUT); //    
				if(actualvoltage_tmp<1000) //   
					{
					actualcurrdata_tmp=(dword)actualvoltage_tmp*10000/actualresdata_tmp; //  
					if(indx_oth.writecurrentdata==TRUE) //     
						{
						ESR_WriteCurrent((word)actualcurrdata_tmp,1); //    eeprom
						indx_oth.writecurrentdata=FALSE; // 
						}
					if(alter_oth.countspotposition==3) actualcurrdata_tmp=(dword)ESR_ReadCurrent(1); //    eeprom     
					}
				else actualcurrdata_tmp=55555; //     55555    
				break;
		case 2: actualresdata_tmp=OTH_ReadResistancedata(FALSE); //   12-18 
				actualpwmdata_tmp=CON_ReadPwmdata(3); //      50 
				actualvoltage_tmp=ADC_AverageSymmdata(FALSE,10,ADCSYMM1X_12INPUT); //    
				if(actualvoltage_tmp<960) //   
					{
					actualcurrdata_tmp=(dword)actualvoltage_tmp*10000/actualresdata_tmp; //  
					if(indx_oth.writecurrentdata==TRUE) //     
						{
						ESR_WriteCurrent((word)actualcurrdata_tmp,2); //    eeprom
						indx_oth.writecurrentdata=FALSE; // 
						}
					if(alter_oth.countspotposition==3) actualcurrdata_tmp=(dword)ESR_ReadCurrent(2); //    eeprom     
					}
				else actualcurrdata_tmp=55555; //     55555    
		}
	OUT_DataPwmcurrcalibrate(actualpwmdata_tmp,(word)actualcurrdata_tmp); //    
	}

void OTH_CorrectPwmdata(bit incdecindx_prm,byte numberpwmdata_prm) //   
	{
	word calibratedata_tmp=0; // 
	calibratedata_tmp=CON_ReadPwmdata(numberpwmdata_prm); //  
	switch(alter_oth.countspotposition) //  
		{
		case 0: asm("nop"); //     
				break;
		case 1: if(incdecindx_prm==FALSE) calibratedata_tmp+=10; //   10   
				else calibratedata_tmp-=10; //   10   
				break;
		case 2: if(incdecindx_prm==FALSE) calibratedata_tmp++; //   1   
				else calibratedata_tmp--; //   1   
				break;
		case 3: asm("nop"); //    
		}
	CON_WritePwmdata(numberpwmdata_prm,calibratedata_tmp); //  
	}

void OTH_ControlPwmcalibrate(byte numberpwmdata_prm,byte readoutcode_prm) //    
	{
	switch(readoutcode_prm) //  
		{
		case 1: if(alter_oth.countspotposition>0) alter_oth.countspotposition--; //         Left
				else alter_oth.countspotposition=3; //  3  
				break;
		case 2: alter_oth.countspotposition++; //      Rigt
				if(alter_oth.countspotposition>3) alter_oth.countspotposition=0; //     3
				break;
		case 3: switch(alter_oth.countspotposition) //     Up
					{
					case 0: asm("nop"); //     
							break;
					case 1 ... 2: OTH_CorrectPwmdata(FALSE,numberpwmdata_prm); //  
							break;
					case 3: asm("nop"); //    
					}
				break;
		case 4: switch(alter_oth.countspotposition) //     Down
					{
					case 0: asm("nop"); //     
							break;
					case 1 ... 2: OTH_CorrectPwmdata(TRUE,numberpwmdata_prm); //  
							break;
					case 3: asm("nop"); //    
					}
				break; 
		case 5: if(indx_oth.writecurrentdata==FALSE) indx_oth.writecurrentdata=TRUE; //         Enter 
		}
	switch(alter_oth.countspotposition) //   
		{
		case 0: alter_oth.spotpositioncursor=0; alter_oth.linepositioncursor=0; //   
				break;
		case 1: alter_oth.spotpositioncursor=7; alter_oth.linepositioncursor=0; //     
				break;
		case 2: alter_oth.spotpositioncursor=8; alter_oth.linepositioncursor=0; //     
				break;
		case 3: alter_oth.spotpositioncursor=12; alter_oth.linepositioncursor=0; //         eeprom
		}
	}
















