/****************************************************
* Resmeasure.c
*     
* : 20.02.2021 18:53:39
* :  .
*****************************************************/

// 
#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/eeprom.h>
#include <math.h>
#include <avr/pgmspace.h>
#include "Main.h"
#include "Systemcontrol.h"
#include "Resmeasure.h"
#include "Symbolcharge.h"
#include "Connectmode.h"
#include "Outmessage.h"

bit_res indx_res={0,0,0,0,0,0,0,0}; // 
var_res alter_res={0,0,0,0,0,0,0,0,0,0,0,0,0,0,0}; // 

EEMEM static word Res_ProbeResistance=RESPROBE_RESISTANCE; //   
EEMEM static word Res_OffsetZero=RESOFFSET_ZERO; //  
EEMEM static dword Res_SupportCapacity=RESSUPPORT_CAPACITY; //   
EEMEM static dword Res_CompConductance=RESCOMP_CONDUCTANCE; //   
EEMEM static dword Res_ThresholdFrequency=RESTHRESHOLD_FREQU; //   

PROGMEM const word Res_CompensationError[25]={2500,2375,2125,1750,1250,1240,1220,1190,1160,1140,1130,1110,1080,1065,1060,1050,1035,1030,1028,1019,1015,1010,1007,1006,1005}; //  

void RES_CycleMeasure(void) //  
	{
	ldword resistancedata_tmp=0; // 
	ldword conductancedata_tmp=0;
	if((indx_res.readydatameasure==FALSE)&&(indx_res.startcyclemeasure==FALSE)) //   
		{
		indx_res.startcyclemeasure=TRUE; //   
		SYS_InitializeVariable(); //   
		SYS_SwitchBlockmode(TRUE); //  
		RES_InicializeVariable(); //  
		SYM_IndicationMeasure(TRUE); //  
		RES_StartstopMeasure(TRUE); //  
		}
	else if((indx_res.readydatameasure==TRUE)&&(indx_res.startcyclemeasure==TRUE)) //   
		{
		RES_StartstopMeasure(FALSE); //  
		SYM_IndicationMeasure(FALSE); //  
		SYS_SwitchBlockmode(FALSE); //  
		indx_res.readydatameasure=FALSE; //   
		indx_res.startcyclemeasure=FALSE; //   
		indx_res.displaydatameasure=TRUE; //   
		if(indx_res.datatimeoverflow==FALSE) resistancedata_tmp=RES_ProcessMeasure(); //     
		else resistancedata_tmp=0; //     
		if(resistancedata_tmp>0) conductancedata_tmp=RES_CalculationConductance(resistancedata_tmp); //      
		else conductancedata_tmp=0; //      
		switch(indx_res.resistanceconductance) // /
			{
			case FALSE: alter_res.measuredataresistance=RES_DecoderLimit(resistancedata_tmp,5); //   
						break;
			case TRUE:  alter_res.measuredataconductance=RES_DecoderLimit(conductancedata_tmp,7); //   
			}
		}
	}

void RES_ReadMeasuredata(bit readydatameasure_prm,bit datatimeoverflow_prm,word countintervalmeasure_prm,dword countoverflowtimer_prm) //   
	{
	indx_res.readydatameasure=readydatameasure_prm; //    
	indx_res.datatimeoverflow=datatimeoverflow_prm; //  
	alter_res.countintervalmeasure=countintervalmeasure_prm; //  
	alter_res.countoverflowtimer=countoverflowtimer_prm; //  
	}

void RES_ConstantaMeasure(void) //   
	{
	floatpoint constantdata_tmp=0; // 
	constantdata_tmp=2*eeprom_read_dword(&Res_SupportCapacity)*log(2); //    
	alter_res.constantaresistance=(ldword)constantdata_tmp; //     
	alter_res.constantafrequency=(ldword)RESFREQU_CRYSTAL*(ldword)RESRATIO_TIMER0*1000; //    
	alter_res.dataoffsetzero=eeprom_read_word(&Res_OffsetZero); //   
	RES_DefineThresholdfrequ(); //  
	alter_res.measuredataresistance=0; //  
	alter_res.measuredatafrequency=0;
	}

void RES_DefineThresholdfrequ(void) //   
	{
	ldword maxresistance_tmp=0; // 
	ldword minresistance_tmp=0;
	ldword maxfrequency_tmp=0;
	ldword minfrequency_tmp=0;
	ldword thresholdfrequency_tmp=0;
	maxresistance_tmp=RESLOAD_RESISTANCE+2*(RESTIME_RESISTANCE+RESALONG_RESISTANCE); //     
	minresistance_tmp=RESLOAD_RESISTANCE+2*(RESTIME_RESISTANCE+53984576); //     15 
	minfrequency_tmp=1e18/(alter_res.constantaresistance/2*maxresistance_tmp); //     
	maxfrequency_tmp=1e18/(alter_res.constantaresistance/2*minresistance_tmp); //     15 
	thresholdfrequency_tmp=(minfrequency_tmp+maxfrequency_tmp)/2; //      
	eeprom_update_dword(&Res_ThresholdFrequency,(dword)thresholdfrequency_tmp); //   
	}

dword RES_DecoderLimit(ldword measurevalue_prm,byte limitingvalue_prm) //   
	{
	ldword measurevalue_tmp; // 
	measurevalue_tmp=measurevalue_prm; //  
	while(measurevalue_tmp>9999) //    
		{
		if(alter_res.countlimitmeasure<limitingvalue_prm) //    
			{
			alter_res.countlimitmeasure++; //  
			measurevalue_tmp/=10; //   10 
			}
		else break; //      
		}
	return (dword)measurevalue_tmp; //  
	}

byte RES_ReadCursorposition(bit indxplaceposition_prm) //       
	{
	byte cursorposition_tmp=0; // 
	switch(indxplaceposition_prm) // / 
		{
		case FALSE: cursorposition_tmp=alter_res.spotpositioncursor; //    
					break;
		case TRUE: cursorposition_tmp=alter_res.linepositioncursor; //   
		}
	return cursorposition_tmp; // / 
	}

void RES_ModeMeasure(bit initializemode_prm) //   
	{
	RES_DisplayModepoints(initializemode_prm); //   
	RES_CycleMeasure(); //  
	}

void RES_DisplayModepoints(bit initializemode_prm) //     
	{
	dword calibratedata_tmp=0; // 
	if(indx_res.displaydatameasure==TRUE) //      
		{
		indx_res.displaydatameasure=FALSE; // 
		if(initializemode_prm==TRUE) indx_res.initializeresmode=TRUE; //      
		if((indx_res.initializeresmode==TRUE)||(alter_res.countlimitmeasure!=alter_res.backuplimitmeasure)) //   
			{
			RES_InicializeLine(); //    
			}
		RES_MeasureDataview(); //  
		switch(alter_res.countlineposition) //   
			{
			case 0 ... 1: if(indx_res.frequencyperiod==FALSE) SYM_OutdataMeasure00000_0(alter_res.measuredatafrequency,7,13,1); //     
					else SYM_OutdataMeasure0000000_0(alter_res.measuredataperiod,7,15,1); //     
					break;
			case 2: calibratedata_tmp=eeprom_read_dword(&Res_SupportCapacity); //   
					SYM_OutdataMeasure00000_0(calibratedata_tmp,13,19,1); //   
					break;
			case 3: calibratedata_tmp=eeprom_read_dword(&Res_CompConductance)/100000; //    
					SYM_OutdataMeasure000_0(calibratedata_tmp,13,17,1); //    
					break;
			case 4: calibratedata_tmp=eeprom_read_word(&Res_ProbeResistance); //   
					SYM_OutdataMeasure00_00(calibratedata_tmp,13,16,1); //    
			}
		indx_res.pinchiptypedevice=SYS_ReadtypePinchip(); //  pin/chip  
		OUT_PinchipType(indx_res.pinchiptypedevice); //  
		if(alter_res.countlimitmeasure>0) alter_res.countlimitmeasure=0; //     
		}
	}

void RES_InicializeLine(void) //      
	{
	if(indx_res.initializeresmode==TRUE) indx_res.initializeresmode=FALSE; // 
	if(alter_res.countlimitmeasure!=alter_res.backuplimitmeasure) alter_res.backuplimitmeasure=alter_res.countlimitmeasure; //      
	switch(indx_res.resistanceconductance) //  /
		{
		case FALSE: switch(alter_res.backuplimitmeasure)  //    
						{
						case 0: SYM_OutcharLine(1,0,0); // : R=   ,  
								break;
						case 1: SYM_OutcharLine(1,1,0); // : R= ,    
								break;
						case 2: SYM_OutcharLine(1,2,0); // : R=  ,   
								break;
						case 3: SYM_OutcharLine(1,3,0); // : R=   ,  
								break;
						case 4: SYM_OutcharLine(1,4,0); // : R= ,    
								break;
						case 5: SYM_OutcharLine(1,5,0); // : R=  ,   
						}
					break;
		case TRUE: switch(alter_res.backuplimitmeasure) //    
						{
						case 0: SYM_OutcharLine(1,6,0); // : G=   ,  
								break;
						case 1: SYM_OutcharLine(1,7,0); // : G= ,    
								break;
						case 2: SYM_OutcharLine(1,8,0); // : G=  ,   
								break;
						case 3: SYM_OutcharLine(1,9,0); // : G=   ,  
								break;
						case 4: SYM_OutcharLine(1,10,0); // : G= ,    
								break;
						case 5: SYM_OutcharLine(1,11,0); // : G=  ,   
								break;
						case 6: SYM_OutcharLine(1,12,0); // : G=   ,  
								break;
						case 7: SYM_OutcharLine(1,13,0); // : G= ,    
						}
		}
	}

void RES_MeasureDataview(void) //   
	{
	switch(indx_res.resistanceconductance) //  /
		{
		case FALSE:	switch(alter_res.backuplimitmeasure) //   
						{
						case 0: SYM_OutdataMeasure000_0(alter_res.measuredataresistance,7,11,0); //    000,0
								break;
						case 1: SYM_OutdataMeasure0_000(alter_res.measuredataresistance,7,9,0); //    0,000
								break;
						case 2: SYM_OutdataMeasure00_00(alter_res.measuredataresistance,7,10,0); //    00,00
								break;
						case 3: SYM_OutdataMeasure000_0(alter_res.measuredataresistance,7,11,0); //    000,0
								break;
						case 4: SYM_OutdataMeasure0_000(alter_res.measuredataresistance,7,9,0); //    0,000
								break;
						case 5: SYM_OutdataMeasure00_00(alter_res.measuredataresistance,7,10,0); //    00,00
						}
					break;
		case TRUE: switch(alter_res.backuplimitmeasure) //   
						{
						case 0: SYM_OutdataMeasure000_0(alter_res.measuredataconductance,7,11,0); //    000,0
								break;
						case 1: SYM_OutdataMeasure0_000(alter_res.measuredataconductance,7,9,0); //    0,000
								break;
						case 2: SYM_OutdataMeasure00_00(alter_res.measuredataconductance,7,10,0); //    00,00
								break;
						case 3: SYM_OutdataMeasure000_0(alter_res.measuredataconductance,7,11,0); //    000,0
								break;
						case 4: SYM_OutdataMeasure0_000(alter_res.measuredataconductance,7,9,0); //    0,000
								break;
						case 5: SYM_OutdataMeasure00_00(alter_res.measuredataconductance,7,10,0); //    00,00
								break;
						case 6: SYM_OutdataMeasure000_0(alter_res.measuredataconductance,7,11,0); //    000,0
								break;
						case 7: SYM_OutdataMeasure0_000(alter_res.measuredataconductance,7,9,0); //    0,000
						}
		}
	}

void RES_InicializeVariable(void) //  
	{
	if(indx_res.readydatameasure==TRUE) indx_res.readydatameasure=FALSE; //      
	if(indx_res.datatimeoverflow==TRUE) indx_res.datatimeoverflow=FALSE; //    
	if(alter_res.countoverflowtimer>0) alter_res.countoverflowtimer=0; //      
	if(alter_res.countintervalmeasure>0) alter_res.countintervalmeasure=0; //      
	}

void RES_StartstopMeasure(bit indxstartstop_prm) //     
	{
	switch(indxstartstop_prm) //  /
		{
		case FALSE: cli(); //  
					ClearBit(TIMSK0,TOIE0); //    0
					ClearBit(TIMSK1,TOIE1); //    1
					break;
		case TRUE: TCNT0=250; // 0
					TCNT1=0; // 1
					TCCR0B=6; //  0      
					SetBit(TIMSK0,TOIE0); //    0
					sei(); //  
		}
	}

ldword RES_ProcessMeasure(void) //    
	{
	ldword resistancedata_tmp=0; // 
	ldword frequencydata_tmp=0;
	frequencydata_tmp=RES_CalculationFrequency();  //  
	if((frequencydata_tmp>eeprom_read_dword(&Res_ThresholdFrequency))&&(alter_res.countoverflowtimer<500)) //   , ..     
		{
		if(SYS_ReadindxOnoffaudio()>0) SYS_OutsignalBeep(1); //   1 ,  
		alter_res.measuredatafrequency=frequencydata_tmp/100; //   
		alter_res.measuredataperiod=RES_CalculationPeriod(frequencydata_tmp); //  
		resistancedata_tmp=RES_CalculationResistance(frequencydata_tmp); //  
		}
	else //   
		{
		if(alter_res.measuredatafrequency>0) alter_res.measuredatafrequency=0; //     
		if(alter_res.measuredataperiod>0) alter_res.measuredataperiod=0; //     
		resistancedata_tmp=0; //  
		}
	return resistancedata_tmp; //  
	}

ldword RES_CalculationFrequency(void) //   
	{
	ldword frequencydata_tmp=0; // 
	ldword countperiodmeasure_tmp=0;
	countperiodmeasure_tmp=RESRATIO_TIMER1*alter_res.countoverflowtimer; //    
	countperiodmeasure_tmp+=(ldword)TCNT1; //   1
	frequencydata_tmp=alter_res.constantafrequency*alter_res.countintervalmeasure/countperiodmeasure_tmp; // 
	return frequencydata_tmp; //  
	}

ldword RES_CalculationResistance(ldword frequencydata_prm) //  
	{
	ldword resistancedata_tmp=0; // 
	ldword intermediatedata_tmp=0;
	intermediatedata_tmp=RES_TotalCalculation(frequencydata_prm); //  
	resistancedata_tmp=intermediatedata_tmp*RESALONG_RESISTANCE; //       
	resistancedata_tmp=resistancedata_tmp/(RESALONG_RESISTANCE-intermediatedata_tmp); //   
	resistancedata_tmp=RES_CorrectMeasure(resistancedata_tmp); //     
	return resistancedata_tmp;  //  
	}

ldword RES_CalculationConductance(ldword resistancedata_prm) //  
	{
	ldword conductancedata_tmp=0; // 
	conductancedata_tmp=100000000000/resistancedata_prm; // 
	return conductancedata_tmp; //  
	}

dword RES_CalculationPeriod(ldword frequencydata_prm) //   
	{
	ldword frequencydata_tmp=0; // 
	frequencydata_tmp=10000000000/frequencydata_prm; // 
	return (dword)frequencydata_tmp; //  
	}

ldword RES_TotalCalculation(ldword frequencydata_prm) //   
	{
	ldword resistancetotal_tmp=0; // 
	resistancetotal_tmp=1000000000000000000/(alter_res.constantaresistance*frequencydata_prm); //   
	resistancetotal_tmp=resistancetotal_tmp-(RESLOAD_RESISTANCE/2+RESTIME_RESISTANCE); //  
	return resistancetotal_tmp; //  
	}

ldword RES_CorrectMeasure(ldword resistancedata_prm) //   
	{
	ldword resistancedata_tmp=0; // 
	ldword conductancedata_tmp;
	alter_res.dataoffsetzero=RES_MeasureOffset(); //  
	if(indx_res.pinchiptypedevice==TRUE) alter_res.dataoffsetzero+=eeprom_read_word(&Res_ProbeResistance); //    chip 
	if(resistancedata_prm>=alter_res.dataoffsetzero) resistancedata_tmp=resistancedata_prm-alter_res.dataoffsetzero; //      
	else resistancedata_tmp=0; //   
	if(resistancedata_tmp>110000000) //   1,1 
		{
		conductancedata_tmp=1000000000000000000/resistancedata_tmp+eeprom_read_dword(&Res_CompConductance); //   
		resistancedata_tmp=1000000000000000000/conductancedata_tmp; //   
		}
	resistancedata_tmp=RES_CompensacionData(resistancedata_tmp)/10; //   
	return resistancedata_tmp; //  
	}

ldword RES_MeasureOffset(void) //   
	{
	ldword frequencydata_tmp=0; // 
	ldword offsetresistance_tmp=0;
	CON_SwitchDevice(TRUE,2); //   
	SYS_TimeWaiting(1000); // 1     
	SYS_InitializeVariable(); //   
	RES_InicializeVariable(); //  
	RES_StartstopMeasure(TRUE); //  
	while(indx_res.readydatameasure==FALSE); //   
	RES_StartstopMeasure(FALSE); //  
	CON_SwitchDevice(FALSE,2); //   
	indx_res.readydatameasure=FALSE; //   
	frequencydata_tmp=RES_CalculationFrequency(); //  
	offsetresistance_tmp=RES_TotalCalculation(frequencydata_tmp); //  
	return offsetresistance_tmp; //   
	}

ldword RES_CompensacionData(ldword resistancedata_prm) //      
	{
	byte indxcoefficient_tmp=0; // 
	ldword resistancedata_tmp=0;
	if(resistancedata_prm<10000) //    100 
		{
		switch(resistancedata_prm) //   
			{
			case 1 ... 10: indxcoefficient_tmp=0; //    0,1 
							break;
			case 11 ... 20: indxcoefficient_tmp=1; //    0,2 
							break;
			case 21 ... 30: indxcoefficient_tmp=2; //    0,3 
							break;
			case 31 ... 40: indxcoefficient_tmp=3; //    0,4 
							break;
			case 41 ... 50: indxcoefficient_tmp=4; //    0,5 
							break;
			case 51 ... 60: indxcoefficient_tmp=5; //    0,6 
							break;
			case 61 ... 70: indxcoefficient_tmp=6; //    0,7 
							break;
			case 71 ... 80: indxcoefficient_tmp=7; //    0,8 
							break;
			case 81 ... 90: indxcoefficient_tmp=8; //    0,9 
							break;
			case 91 ... 100: indxcoefficient_tmp=9; //    1,0 
							break;
			case 101 ... 200: indxcoefficient_tmp=10; //    2,0 
							break;
			case 201 ... 300: indxcoefficient_tmp=11; //    3,0 
							break;
			case 301 ... 400: indxcoefficient_tmp=12; //    4,0 
							break;
			case 401 ... 500: indxcoefficient_tmp=13; //    5,0 
							break;
			case 501 ... 600: indxcoefficient_tmp=14; //    6,0 
							break;
			case 601 ... 700: indxcoefficient_tmp=15; //    7,0 
							break;
			case 701 ... 800: indxcoefficient_tmp=16; //    8,0 
							break;
			case 801 ... 900: indxcoefficient_tmp=17; //    9,0 
							break;
			case 901 ... 1000: indxcoefficient_tmp=18; //    10,0 
							break;
			case 1001 ... 2000: indxcoefficient_tmp=19; //    20,0 
							break;
			case 2001 ... 3000: indxcoefficient_tmp=20; //    30,0 
							break;
			case 3001 ... 4000: indxcoefficient_tmp=21; //    40,0 
							break;
			case 4001 ... 6000: indxcoefficient_tmp=22; //    60,0 
							break;
			case 6001 ... 8000: indxcoefficient_tmp=23; //    80,0 
							break;
			case 8001 ... 9999: indxcoefficient_tmp=24; //    100,0 
			}
		resistancedata_tmp=resistancedata_prm*(ldword)pgm_read_word(&Res_CompensationError[indxcoefficient_tmp])/1000; // 
		}
	else resistancedata_tmp=resistancedata_prm; //      100 
	return resistancedata_tmp; //  
	}

void RES_ControlMode(byte readoutcode_prm) //      
	{
	switch(readoutcode_prm) //   
		{
		case 1: switch(alter_res.countlineposition) //       Left
					{
					case 0: if(alter_res.countspotposition==0) alter_res.countspotposition=1; //    
							else alter_res.countspotposition=0; //    
							break;
					case 1: asm("nop"); // 
							break;
					case 2: if(alter_res.countspotposition>1) alter_res.countspotposition--; //         1
							else alter_res.countspotposition=5; //  5     1
							break;
					case 3: if(alter_res.countspotposition>1) alter_res.countspotposition--; //         1
							else alter_res.countspotposition=4; //  4     1
							break;
					case 4: if(alter_res.countspotposition>1) alter_res.countspotposition--; //          1
							else alter_res.countspotposition=5; //  5     1
					}
				break;
		case 2: switch(alter_res.countlineposition) //       Rigt
					{
					case 0: if(alter_res.countspotposition==0) alter_res.countspotposition=1; //    
							else alter_res.countspotposition=0; //    
							break;
					case 1: asm("nop"); //     
							break;
					case 2: alter_res.countspotposition++; //      
							if(alter_res.countspotposition>5) alter_res.countspotposition=1; //  1     5
							break;
					case 3: alter_res.countspotposition++; //      
							if(alter_res.countspotposition>4) alter_res.countspotposition=1; //  1     4
							break;
					case 4: alter_res.countspotposition++; //       
							if(alter_res.countspotposition>5) alter_res.countspotposition=1; //  1     5
					}
				break;
		case 3: if(alter_res.countspotposition==1) //        Up
					{
					alter_res.countlineposition++; //  
					if(alter_res.countlineposition>4) alter_res.countlineposition=0; //    4
					RES_UpdateLine(); //  
					}
				else RES_CorrectCalibrate(FALSE); //   
				break;
		case 4: if(alter_res.countspotposition==1) //        Down
					{
					if(alter_res.countlineposition>0) alter_res.countlineposition--; //     
					else alter_res.countlineposition=4; //   4  
					RES_UpdateLine(); //  
					}
				else RES_CorrectCalibrate(TRUE); //   
				break;
		case 5: switch(alter_res.countlineposition) //       Enter
					{
					case 0: if(indx_res.resistanceconductance==FALSE) indx_res.resistanceconductance=TRUE; //    
							else indx_res.resistanceconductance=FALSE; //    
							indx_res.initializeresmode=TRUE; //      
							indx_res.displaydatameasure=TRUE; //   
							break;
					case 1: if(indx_res.frequencyperiod==FALSE) //    
								{
								indx_res.frequencyperiod=TRUE; //    
								SYM_OutcharLine(0,4,1); // : T=    ,  
								}
							else //    
								{
								indx_res.frequencyperiod=FALSE; //    
								SYM_OutcharLine(0,3,1); // : F=     ,  
								}
							break;
					case 2: asm("nop"); // 
							break;
					case 3: asm("nop"); // 
							break;
					case 4: asm("nop"); // 
					}
		}
	switch(alter_res.countlineposition) //   
		{
		case 0: alter_res.linepositioncursor=0; //   
				switch(alter_res.countspotposition) //   
					{
					case 0: alter_res.spotpositioncursor=0; //        
							break;
					case 1: alter_res.spotpositioncursor=5; //         /
					}
				break;
		case 1: alter_res.linepositioncursor=1; //   
				alter_res.spotpositioncursor=5; //      /
				break;
		case 2:	alter_res.linepositioncursor=1; //   
				switch(alter_res.countspotposition) //     
					{
					case 1: alter_res.spotpositioncursor=9; //     
							break;
					case 2: alter_res.spotpositioncursor=14; //     
							break;
					case 3: alter_res.spotpositioncursor=15; //     
							break;
					case 4: alter_res.spotpositioncursor=16; //     
							break;
					case 5: alter_res.spotpositioncursor=17; //     
					}
				break;
		case 3: alter_res.linepositioncursor=1; //   
				switch(alter_res.countspotposition) //       
					{
					case 1: alter_res.spotpositioncursor=9; //     
							break;
					case 2: alter_res.spotpositioncursor=13; //     
							break;
					case 3: alter_res.spotpositioncursor=14; //     
							break;
					case 4: alter_res.spotpositioncursor=15; //     
					}
				break;
		case 4: alter_res.linepositioncursor=1; //   
		switch(alter_res.countspotposition) //       
			{
			case 1: alter_res.spotpositioncursor=9; //       
					break;
			case 2: alter_res.spotpositioncursor=13; //     
					break;
			case 3: alter_res.spotpositioncursor=14; //     
					break;
			case 4: alter_res.spotpositioncursor=16; //      
					break;
			case 5: alter_res.spotpositioncursor=17; //      
			}
		}
	}

void RES_UpdateLine(void) //    
	{
	switch(alter_res.countlineposition) //   
		{
		case 0 ... 1: if(indx_res.frequencyperiod==FALSE) SYM_OutcharLine(0,3,1); // : F=     ,  
				else SYM_OutcharLine(0,4,1); // : T=       ,  
				break;
		case 2: SYM_OutcharLine(1,14,1); // : .Co      ,  
				break;
		case 3: SYM_OutcharLine(1,15,1); // : .Gk    ,  
				break;
		case 4: SYM_OutcharLine(1,16,1); // : .Rp   ,   
		}
	}

void RES_CorrectCalibrate(bit indxincdecdata_prm) //   
	{
	switch(alter_res.countlineposition) //   
		{
		case 0 ... 1: asm("nop"); // 
				break;
		case 2:	RES_CorrectCapacity(indxincdecdata_prm); //   
				RES_ConstantaMeasure(); // 
				break;
		case 3: RES_CorrectConductance(indxincdecdata_prm); //   
				break;
		case 4: RES_CorrectProbe(indxincdecdata_prm); //    
		}
	}

void RES_CorrectCapacity(bit indxincdecdata_prm) //    
	{
	dword capacitydata_tmp=0; // 
	capacitydata_tmp=eeprom_read_dword(&Res_SupportCapacity); //   
	switch(alter_res.countspotposition) //   
		{
		case 1: asm("nop"); // 
				break;
		case 2: if(indxincdecdata_prm==FALSE) capacitydata_tmp+=10000; //   10000   
				else if((indxincdecdata_prm==TRUE)&&(capacitydata_tmp>10000)) capacitydata_tmp-=10000; //   10000   
				break;
		case 3: if(indxincdecdata_prm==FALSE) capacitydata_tmp+=1000; //   1000   
				else if((indxincdecdata_prm==TRUE)&&(capacitydata_tmp>1000)) capacitydata_tmp-=1000; //   1000   
				break;
		case 4: if(indxincdecdata_prm==FALSE) capacitydata_tmp+=100; //   100   
				else if((indxincdecdata_prm==TRUE)&&(capacitydata_tmp>100)) capacitydata_tmp-=100; //   100   
				break;
		case 5: if(indxincdecdata_prm==FALSE) capacitydata_tmp+=10; //   10   
				else if((indxincdecdata_prm==TRUE)&&(capacitydata_tmp>10)) capacitydata_tmp-=10; //   10   
		}
	eeprom_update_dword(&Res_SupportCapacity,capacitydata_tmp); //   
	}

void RES_CorrectConductance(bit indxincdecdata_prm) //     
	{
	dword conductancedata_tmp=0; // 
	conductancedata_tmp=eeprom_read_dword(&Res_CompConductance); //  
	switch(alter_res.countspotposition) //   
		{
		case 1: asm("nop"); // 
				break;
		case 2: if(indxincdecdata_prm==FALSE) conductancedata_tmp+=100000000; //   100000000   
				else if((indxincdecdata_prm==TRUE)&&(conductancedata_tmp>100000000)) conductancedata_tmp-=100000000; //   100000000   
				break;
		case 3: if(indxincdecdata_prm==FALSE) conductancedata_tmp+=10000000; //   10000000   
				else if((indxincdecdata_prm==TRUE)&&(conductancedata_tmp>10000000)) conductancedata_tmp-=10000000; //   10000000   
				break;
		case 4: if(indxincdecdata_prm==FALSE) conductancedata_tmp+=1000000; //   1000000   
				else if((indxincdecdata_prm==TRUE)&&(conductancedata_tmp>1000000)) conductancedata_tmp-=1000000; //   1000000   
		}
	eeprom_update_dword(&Res_CompConductance,conductancedata_tmp); //  
	}

void RES_CorrectProbe(bit indxincdecdata_prm) //     
	{
	word resistancedata_tmp=0; // 
	resistancedata_tmp=eeprom_read_word(&Res_ProbeResistance); //    
	switch(alter_res.countspotposition) //   
		{
		case 1: asm("nop"); // 
				break;
		case 2: if(indxincdecdata_prm==FALSE) resistancedata_tmp+=1000; //   1000   
				else if((indxincdecdata_prm==TRUE)&&(resistancedata_tmp>1000)) resistancedata_tmp-=1000; //   1000   
				break;
		case 3: if(indxincdecdata_prm==FALSE) resistancedata_tmp+=100; //   100   
				else if((indxincdecdata_prm==TRUE)&&(resistancedata_tmp>100)) resistancedata_tmp-=100; //   100   
				break;
		case 4: if(indxincdecdata_prm==FALSE) resistancedata_tmp+=10; //   10   
				else if((indxincdecdata_prm==TRUE)&&(resistancedata_tmp>10)) resistancedata_tmp-=10; //   10   
				break;
		case 5: if(indxincdecdata_prm==FALSE) resistancedata_tmp+=1; //   1   
				else if((indxincdecdata_prm==TRUE)&&(resistancedata_tmp>1)) resistancedata_tmp-=1; //   1   
		}
	eeprom_update_word(&Res_ProbeResistance,resistancedata_tmp); //   
	}

dword RES_ReadProberesistance(void) //    
	{
	dword proberesistance_tmp=0; // 
	proberesistance_tmp=(dword)eeprom_read_word(&Res_ProbeResistance); //   
	return proberesistance_tmp*10; //    
	}



















