/*********************************
* Systemcontrol.c
*     
* : 15.01.2021 22:17:15
* :  .
**********************************/ 

// 
#include <avr/io.h>
#include <util/delay_basic.h>
#include <avr/interrupt.h>
#include <avr/eeprom.h>
#include "Main.h"
#include "Systemcontrol.h"
#include "Keybuttscan.h"
#include "Symbolcharge.h"
#include "Connectmode.h"
#include "Outmessage.h"
#include "Resmeasure.h"
#include "Cappmeasure.h"
#include "Capmeasure.h"
#include "Indmeasure.h"
#include "Adconverter.h"
#include "Othercalibrations.h"
#include "Lcdworkunit.h"

bit_sys indx_sys={0,0,1,0,0,0,0}; // 
var_sys alter_sys={0,4,0,0,0,0}; // 

EEMEM static byte Sys_IndxonoffAudio=SYSONOFF_AUDIO; //  /   
EEMEM static word Sys_MaschineCycle=0; //      
EEMEM static word Sys_HalfperiodBeep=0; //        
EEMEM static dword Sys_FrequencyPiezo=SYSFREQU_PIEZO; //  

ISR(TIMER0_OVF_vect) //     0
	{
	if(indx_sys.synchronization==TRUE) //   
		{
		alter_sys.countintervalmeasure++; //   
		if(alter_sys.countlimitmeasuretime>99)  SYS_StoppingMeasure(); //        
		}
	else //   
		{
		TCCR1B=1; //  1    
		if(bit_is_set(TIFR1,TOV1)) SetBit(TIFR1,TOV1); //   1  
		SetBit(TIMSK1,TOIE1); //    1
		indx_sys.synchronization=TRUE; //  
		}
	}

ISR(TIMER1_OVF_vect) //     1
	{
	alter_sys.countlimitmeasuretime++; //   1
	if(alter_sys.countlimitmeasuretime>=500)  SYS_StoppingMeasure(); //        
	}

void SYS_StoppingMeasure(void) //   
	{
	TCCR1B=0; //  1
	TCCR0B=0; //  0
	indx_sys.readydatameasure=TRUE; //    
	if(alter_sys.countlimitmeasuretime>=500) indx_sys.datatimeoverflow=TRUE; //       
	switch(alter_sys.countmodemeasure) //     
		{
		case 0: RES_ReadMeasuredata(indx_sys.readydatameasure,indx_sys.datatimeoverflow,alter_sys.countintervalmeasure,alter_sys.countlimitmeasuretime); //   
				break;
		case 1: asm("nop"); // 
				break;
		case 2: CAP_ReadMeasure(indx_sys.readydatameasure,indx_sys.datatimeoverflow,alter_sys.countintervalmeasure,alter_sys.countlimitmeasuretime); //   
				break;
		case 3: IND_ReadMeasuredata(indx_sys.readydatameasure,indx_sys.datatimeoverflow,alter_sys.countintervalmeasure,alter_sys.countlimitmeasuretime); //   
		}
	}

void SYS_InitializeVariable(void) //  
	{
	if(indx_sys.synchronization==TRUE) indx_sys.synchronization=FALSE; //    
	if(indx_sys.readydatameasure==TRUE) indx_sys.readydatameasure=FALSE; //      
	if(indx_sys.datatimeoverflow==TRUE) indx_sys.datatimeoverflow=FALSE; //    
	if(alter_sys.countlimitmeasuretime>0) alter_sys.countlimitmeasuretime=0; //      
	if(alter_sys.countintervalmeasure>0) alter_sys.countintervalmeasure=0; //      
	}

void SYS_TimeWaiting(word milliseconds_prm) //    
	{
	word milliseconds_tmp; // 
	milliseconds_tmp=milliseconds_prm; //  
	while(milliseconds_tmp>0) //   
		{
		_delay_loop_2(4000); //  1    
		milliseconds_tmp--; //  
		}
	}

void SYS_InitializeBeep(void) //   
	{
	dword halfperiodbeep_tmp; // 
	dword maschinecycle_tmp;
	if(eeprom_read_word(&Sys_HalfperiodBeep)==0) //   
		{
		maschinecycle_tmp=400000/(SYSFREQU_CRYSTAL/1000); //      
		halfperiodbeep_tmp=50000000/eeprom_read_dword(&Sys_FrequencyPiezo); //    
		halfperiodbeep_tmp/=maschinecycle_tmp; //       
		eeprom_write_word(&Sys_MaschineCycle,(word)maschinecycle_tmp); //       
		eeprom_write_word(&Sys_HalfperiodBeep,(word)halfperiodbeep_tmp); //        
		}
	}

void SYS_OutsignalBeep(byte durationofbeep_prm) //       
	{
	word halfperiodbeep_tmp=0; // 
	dword countduration_tmp;
	halfperiodbeep_tmp=eeprom_read_word(&Sys_HalfperiodBeep); //    
	countduration_tmp=durationofbeep_prm*50000/halfperiodbeep_tmp/eeprom_read_word(&Sys_MaschineCycle); //   
	while(countduration_tmp>0) //   
		{
		SetBit(SYSCONTROL_BEEP,SYSOUTBIT_BEEP); //       
		_delay_loop_2(halfperiodbeep_tmp); //      
		ClearBit(SYSCONTROL_BEEP,SYSOUTBIT_BEEP); //       
		_delay_loop_2(halfperiodbeep_tmp); //      
		countduration_tmp--; //    
		}
	}

void SYS_MainMode(void) //  
	{
	bit initializemode_tmp=FALSE; // 
	byte datacodemode_tmp=0;
	LCD_WriteCommon(14); //  
	datacodemode_tmp=SYS_ControlDecoder(); //  
	indx_sys.pinchiptype=KEY_PinchipType(indx_sys.pinchiptype); //   pin/chip
	if(indx_sys.measurecalibrate==FALSE) //     
		{
		if(indx_sys.initializemode==TRUE) //    
			{
			SYS_InitializeMeasure(datacodemode_tmp); //   
			initializemode_tmp=TRUE; //    
			if(alter_sys.countmodemeasure!=datacodemode_tmp) alter_sys.countmodemeasure=datacodemode_tmp; //   
			indx_sys.initializemode=FALSE; // 
			}
		switch(datacodemode_tmp) //  
			{
			case 0: RES_ModeMeasure(initializemode_tmp); //     
					break;
			case 1: CAPP_ModeMeasure(initializemode_tmp); //      1   
					break;
			case 2: CAP_ModeMeasure(initializemode_tmp); //      1 
					break;
			case 3: IND_ModeMeasure(initializemode_tmp); //     
			}
		}
	else //     
		{
		if(indx_sys.initializemode==TRUE) //    
			{
			OTH_InitializeCalibrate(datacodemode_tmp); //   
			if(alter_sys.countmodecalibrate!=datacodemode_tmp) alter_sys.countmodecalibrate=datacodemode_tmp; //    
			indx_sys.initializemode=FALSE; // 
			}
		OTH_ViewCalibratevalue(datacodemode_tmp); //   
		}
	OUT_CursorPosition(alter_sys.spotpositioncursor,alter_sys.linepositioncursor); // 
	SYS_TimeWaiting(150); //   150 
	LCD_WriteCommon(12); //  
	SYS_TimeWaiting(150); //   150 
	}

byte SYS_ControlDecoder(void) //   
	{
	byte readoutcode_tmp=0; // 
	byte numdatamode_tmp=0;
	if(indx_sys.blockswitchmode==FALSE) readoutcode_tmp=KEY_ScanButton(); //       
	if(readoutcode_tmp>0) //  
		{
		SYS_OutsignalBeep(20); //    20 
		if((alter_sys.spotpositioncursor==0)&&(alter_sys.linepositioncursor==0)&&(readoutcode_tmp>2)) //         
			{
			numdatamode_tmp=SYS_ControlMode(readoutcode_tmp); //    
			}
		else //     
			{
			if(indx_sys.measurecalibrate==FALSE) numdatamode_tmp=alter_sys.countmodemeasure; //    
			else numdatamode_tmp=alter_sys.countmodecalibrate; //     
			switch(numdatamode_tmp) //    
				{
				case 0: RES_ControlMode(readoutcode_tmp); //      
						alter_sys.spotpositioncursor=RES_ReadCursorposition(FALSE); //     
						alter_sys.linepositioncursor=RES_ReadCursorposition(TRUE); //     
						break;
				case 1: CAPP_ControlMode(readoutcode_tmp); //       1   
						alter_sys.spotpositioncursor=CAPP_ReadCursorposition(FALSE); //     
						alter_sys.linepositioncursor=CAPP_ReadCursorposition(TRUE); //     
						break;
				case 2: CAP_ControlMode(readoutcode_tmp); //        1 
						alter_sys.spotpositioncursor=CAP_ReadCursorposition(FALSE); //     
						alter_sys.linepositioncursor=CAP_ReadCursorposition(TRUE); //     
						break;
				case 3: IND_ControlModemeasure(readoutcode_tmp); //      
						alter_sys.spotpositioncursor=IND_ReadCursorposition(FALSE); //     
						alter_sys.linepositioncursor=IND_ReadCursorposition(TRUE); //     
						break;
				case 4 ... 11: OTH_ControlModecalibrate(numdatamode_tmp,readoutcode_tmp); //     
						alter_sys.spotpositioncursor=OTH_ReadCursorposition(FALSE); //     
						alter_sys.linepositioncursor=OTH_ReadCursorposition(TRUE); //     
				}
			}
		}
	else //   
		{
		if(indx_sys.measurecalibrate==FALSE) numdatamode_tmp=alter_sys.countmodemeasure; //    
		else numdatamode_tmp=alter_sys.countmodecalibrate; //     
		}
	return numdatamode_tmp; //    
	}

byte SYS_ControlMode(byte readoutcode_prm) //      
	{
	byte countdatamode_tmp=0; // 
	switch(readoutcode_prm) //   
		{
		case 3:	if(indx_sys.measurecalibrate==FALSE) //        Up
					{
					countdatamode_tmp=alter_sys.countmodemeasure; //   
					countdatamode_tmp++; //   
					if(countdatamode_tmp>3) countdatamode_tmp=0; //     
					}
				else //        Up
					{
					countdatamode_tmp=alter_sys.countmodecalibrate; //    
					countdatamode_tmp++; //   
					if(countdatamode_tmp>11) countdatamode_tmp=4; //      
					}
				indx_sys.initializemode=TRUE; //     
				break;
		case 4: if(indx_sys.measurecalibrate==FALSE) //        Down
					{
					countdatamode_tmp=alter_sys.countmodemeasure; //    
					if(countdatamode_tmp>0) countdatamode_tmp--; //      
					else countdatamode_tmp=3; //      
					}
				else //        Down
					{
					countdatamode_tmp=alter_sys.countmodecalibrate; //    
					if(countdatamode_tmp>4) countdatamode_tmp--; //      
					else countdatamode_tmp=11; //       
					}
				indx_sys.initializemode=TRUE; //     
				break;
		case 5: if(indx_sys.measurecalibrate==FALSE) //        Enter
					{
					indx_sys.measurecalibrate=TRUE; //      
					countdatamode_tmp=4; //    
					}
				else //        Enter
					{
					indx_sys.measurecalibrate=FALSE; //      
					countdatamode_tmp=0; //    
					}
				indx_sys.initializemode=TRUE; //    
		}
	return countdatamode_tmp; //     
	}

void SYS_InitializeMeasure(byte datacodemode_prm) //    
	{
	switch(datacodemode_prm) //  
		{
		case 0:	SYM_ResGraphics(); //     
				RES_ConstantaMeasure(); // 
				CON_ResistanceMode(); // 
				OUT_ResistanceTitle(); //   
				OUT_PinchipType(indx_sys.pinchiptype); //  
				break;
		case 1: SYM_CappGraphics(); //      1   
				CAPP_ConstantaMeasure(); // 
				CON_CappacityMode(); // 
				OUT_CappacityTitle(); //   
				OUT_PinchipType(indx_sys.pinchiptype); //  
				break;
		case 2: SYM_CapGraphics(); //       1 
				CAP_ConstantaMeasure(); // 
				CON_CapacityMode(); // 
				OUT_CapacityTitle(); //   
				OUT_PinchipType(indx_sys.pinchiptype); //  
				break;
		case 3: SYM_IndGraphics(); //     
				IND_ConstantaMeasure(); // 
				CON_InductanceMode(); // 
				OUT_InductanceTitle(); //   
				OUT_PinchipType(indx_sys.pinchiptype); //  
		}
	}

bit SYS_ReadtypePinchip(void) //   pin/chip  
	{
	return indx_sys.pinchiptype; //  
	}

void SYS_SwitchBlockmode(bit indxswitchdata_prm) //     
	{
	indx_sys.blockswitchmode=indxswitchdata_prm; //    
	}

dword SYS_ReadFrequencypiezo(void) //     
	{
	return eeprom_read_dword(&Sys_FrequencyPiezo); //   
	}

void SYS_WriteFrequencypiezo(dword frequencypiezodata_prm) //     
	{
	eeprom_update_dword(&Sys_FrequencyPiezo,frequencypiezodata_prm); //   
	}

byte SYS_ReadindxOnoffaudio(void) //   /   
	{
	return eeprom_read_byte(&Sys_IndxonoffAudio); //  
	}

void SYS_WriteindxOnoffaudio(byte indxonoffdata_prm) //   /   
	{
	eeprom_update_byte(&Sys_IndxonoffAudio,indxonoffdata_prm); //  
	}
















